import os
import argparse


stats_arcog_usage = '''
========================== stats_arcog example commands ==========================

BioSAK stats_arcog -cog cog.txt -o op_dir -db /Users/songweizhi/DB/arCOG18

# Required DB files:
arCOGdef.tab: https://ftp.ncbi.nih.gov/pub/wolf/COGs/arCOG/tmp.ar18/arCOGdef.tab
fun-20.tab:   https://ftp.ncbi.nih.gov/pub/COG/COG2020/data/fun-20.tab

==================================================================================
'''


def stats_arcog(args):

    cog_txt             = args['cog']
    op_dir              = args['o']
    db_dir              = args['db']
    force_create_op_dir = args['f']

    ####################################################################################################################

    cog_des_txt     = '%s/arCOGdef.tab'         % db_dir
    pwd_fun_20_tab  = '%s/fun-20.tab'           % db_dir
    op_txt          = '%s/COG_cate.txt'         % op_dir
    op_stats_txt    = '%s/COG_cate_stats.txt'   % op_dir

    if os.path.isdir(op_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    ############################################## Read in KEGG DB files ###############################################

    print('Read in arCOG DB files')

    cog_id_to_category_dict = dict()
    cog_id_to_description_dict = dict()
    for each_cog in open(cog_des_txt, encoding="ISO-8859-1"):
        each_cog_split = each_cog.strip().split('\t')
        cog_id = each_cog_split[0]
        cog_cate_str = each_cog_split[1]
        cog_cate_split = [i for i in cog_cate_str]
        cog_desc = each_cog_split[3]
        cog_id_to_description_dict[cog_id] = cog_desc
        cog_id_to_category_dict[cog_id] = cog_cate_split

    cog_category_to_description_dict = {}
    for cog_category in open(pwd_fun_20_tab):
        cog_category_split = cog_category.strip().split('\t')
        cog_category_to_description_dict[cog_category_split[0]] = cog_category_split[2]

    ########################################################################################################################

    input_cog_id_set = set()
    for each_cog in open(cog_txt):
        cog_id = each_cog.strip().split('\t')[0]
        if cog_id.startswith('arCOG'):
            input_cog_id_set.add(cog_id)

    cog_cate_stats_dict = dict()
    for cog_id in input_cog_id_set:
        cog_cate_list = cog_id_to_category_dict[cog_id]
        for cog_cate in cog_cate_list:
            if cog_cate not in cog_cate_stats_dict:
                cog_cate_stats_dict[cog_cate] = set()
            cog_cate_stats_dict[cog_cate].add(cog_id)

    op_txt_handle = open(op_txt, 'w')
    op_txt_handle.write('Category\tCategory_description\tCOG\tDescription\n')
    op_stats_txt_handle = open(op_stats_txt, 'w')
    for cog_cate in sorted(list(cog_cate_stats_dict.keys())):
        cog_id_set = cog_cate_stats_dict[cog_cate]
        cog_cate_desc = cog_category_to_description_dict[cog_cate]
        op_stats_txt_handle.write('%s\t%s\t%s\n' % (cog_cate, len(cog_id_set), cog_cate_desc))
        for cog_id in sorted(list(cog_id_set)):
            cog_id_desc = cog_id_to_description_dict[cog_id]
            op_txt_handle.write('%s\t%s\t%s\t%s\t%s\n' % (cog_cate, len(cog_id_set), cog_cate_desc.strip(), cog_id, cog_id_desc.strip()))
    op_txt_handle.close()
    op_stats_txt_handle.close()

    print('Done!')


if __name__ == "__main__":

    stats_arcog_parser = argparse.ArgumentParser(usage=stats_arcog_usage)
    stats_arcog_parser.add_argument('-cog',         required=True,                          help='cog_ids.txt')
    stats_arcog_parser.add_argument('-db',          required=True,                          help='DB folder, contains arCOGdef.tab and fun-20.tab')
    stats_arcog_parser.add_argument('-o',           required=True,                          help='output directory')
    stats_arcog_parser.add_argument('-f',           required=False, action="store_true",    help='force overwrite')
    args = vars(stats_arcog_parser.parse_args())
    stats_arcog(args)



import os
import argparse
from Bio import SeqIO
from datetime import datetime
from BioSAK.global_functions import time_format


select_seq_usage = '''
========================= select_seq example commands =========================

BioSAK select_seq -i input.fa -id seq_id.txt -o output.fa
BioSAK select_seq -i input.fa -id seq_id.txt -o output.fa -exclude

# ID file format: 
one id per line, do not include great than (>) symbol.

===============================================================================
'''

def select_seq(args):

    # read in argument
    seq_file        = args['i']
    id_file         = args['id']
    output_file     = args['o']
    exclude_seqs    = args['exclude']
    one_line        = args['oneline']
    in_fastq        = args['fq']

    # get provided id list
    print(datetime.now().strftime(time_format) + 'Read in sequence id in %s' % id_file)
    seq_id_list = set()
    for seq_id in open(id_file):
        seq_id_list.add(seq_id.strip())

    seq_in_format = 'fasta'
    if in_fastq is True:
        seq_in_format = 'fastq'

    # report
    if exclude_seqs is False:
        print(datetime.now().strftime(time_format) + 'Extracting sequences in %s' % id_file)
    else:
        print(datetime.now().strftime(time_format) + 'Extracting sequences except those in %s' % id_file)

    # extract sequences
    output_file_handle = open(output_file, 'w')
    for seq_record in SeqIO.parse(seq_file, seq_in_format):
        seq_id = seq_record.id
        if exclude_seqs is False:
            if seq_id in seq_id_list:

                if in_fastq is False:
                    if one_line is False:
                        SeqIO.write(seq_record, output_file_handle, 'fasta')
                    else:
                        SeqIO.write(seq_record, output_file_handle, 'fasta-2line')
                else:
                    SeqIO.write(seq_record, output_file_handle, 'fastq')

        else:
            if seq_id not in seq_id_list:

                if in_fastq is False:
                    if one_line is False:
                        SeqIO.write(seq_record, output_file_handle, 'fasta')
                    else:
                        SeqIO.write(seq_record, output_file_handle, 'fasta-2line')
                else:
                    SeqIO.write(seq_record, output_file_handle, 'fastq')

    output_file_handle.close()

    print(datetime.now().strftime(time_format) + 'Sequences exported to %s' % output_file)
    print(datetime.now().strftime(time_format) + 'Done!')


if __name__ == '__main__':

    select_seq_parser = argparse.ArgumentParser(usage=select_seq_usage)
    select_seq_parser.add_argument('-i',       required=True,                          help='fasta file')
    select_seq_parser.add_argument('-id',       required=True,                          help='sequence id,one id per line')
    select_seq_parser.add_argument('-o',       required=True,                          help='output file')
    select_seq_parser.add_argument('-exclude', required=False, action="store_true",    help='specify to extract sequences except those in -id')
    select_seq_parser.add_argument('-fq',      required=False, action="store_true",    help='in fastq format, default: fa')
    select_seq_parser.add_argument('-oneline', required=False, action="store_true",    help='put sequence in single line')
    args = vars(select_seq_parser.parse_args())
    select_seq(args)



import os
import glob
import math
import argparse
import subprocess
from Bio import SeqIO
import multiprocessing as mp


Usearch16S_usage = '''
====================================== Usearch16S example commands ======================================

# This is a wrapper for the following Usearch modules, please refer to the output cmds.txt for details:
-fastx_uniques, -unoise3, -uchime2_ref, -otutab

# classify OTU with best hit approach
BioSAK Usearch16S -i CleanData -x fna -o op_dir -t 12 -f -r ssu_all_r220.blca.fa
BioSAK Usearch16S -i CleanData -x fna -o op_dir -t 12 -f -r SILVA_138.2_SSURef_NR99_tax_silva.blca.fa

# classify OTU with BLCA
BioSAK Usearch16S -i CleanData -x fna -o op_dir -t 12 -f -blca -r ssu_all_r220.blca.fa -c ssu_all_r220.blca.tax
BioSAK Usearch16S -i CleanData -x fna -o op_dir -t 12 -f -blca -r SILVA_138.2_SSURef_NR99_tax_silva.blca.fa -c SILVA_138.2_SSURef_NR99_tax_silva.blca.tax

# GTDB SSU (BLCA compatible)
-r /Users/songweizhi/DB/BLCA/GTDB_SSU/ssu_all_r220.blca.fa
-c /Users/songweizhi/DB/BLCA/GTDB_SSU/ssu_all_r220.blca.tax

# SILVA SSU (BLCA compatible)
-r /Users/songweizhi/DB/BLCA/SILVA_SSU/SILVA_138.2_SSURef_NR99_tax_silva.blca.fa
-c /Users/songweizhi/DB/BLCA/SILVA_SSU/SILVA_138.2_SSURef_NR99_tax_silva.blca.tax

=========================================================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def check_executables(program_list):

    not_detected_programs = []
    for needed_program in program_list:

        if subprocess.call(['which', needed_program], stdout=open(os.devnull, 'wb')) != 0:
            not_detected_programs.append(needed_program)

    if not_detected_programs != []:
        print('%s not detected, program exited!' % ','.join(not_detected_programs))
        exit()


def best_hit(file_in, file_out):

    file_out_handle = open(file_out, 'w')
    best_hit_line = ''
    best_hit_query_id = ''
    best_hit_score = 0
    for blast_hit in open(file_in):
        blast_hit_split = blast_hit.strip().split('\t')
        query_id = blast_hit_split[0]
        bit_score = float(blast_hit_split[11])

        if best_hit_query_id == '':
            best_hit_query_id = query_id
            best_hit_line = blast_hit
            best_hit_score = bit_score

        elif (query_id == best_hit_query_id) and (bit_score > best_hit_score):
            best_hit_score = bit_score
            best_hit_line = blast_hit

        elif query_id != best_hit_query_id:
            file_out_handle.write(best_hit_line)
            best_hit_query_id = query_id
            best_hit_line = blast_hit
            best_hit_score = bit_score

    file_out_handle.write(best_hit_line)
    file_out_handle.close()


def parse_blca_op(blca_output):

    blca_op_name, blca_op_path, blca_op_base, blca_op_ext = sep_path_basename_ext(blca_output)
    output_file_1 = '%s/%s.1.txt' % (blca_op_path, blca_op_base)
    output_file_2 = '%s/%s.2.txt' % (blca_op_path, blca_op_base)

    # read in input file
    s16_taxon_blca_dict = {}
    for each_16s_taxon in open(blca_output):
        each_16s_taxon_split = each_16s_taxon.strip().split('\t')
        s16_taxon_blca_dict[each_16s_taxon_split[0]] = each_16s_taxon_split[1]

    taxon_dict_formatted_with_num = {}
    taxon_dict_formatted_no_num = {}
    for each_16s in s16_taxon_blca_dict:
        taxon_blca_raw = s16_taxon_blca_dict[each_16s]
        formatted_taxon_str_with_num = 'Unclassified'
        formatted_taxon_str_no_num = 'Unclassified'
        if taxon_blca_raw != 'Unclassified':
            split2 = taxon_blca_raw.strip().split(';')[:-1]
            formatted_taxon_list_with_num2 = []
            formatted_taxon_list_no_num2 = []
            element_index = 0
            while element_index < len(split2):
                taxon_blca = split2[element_index]
                confidence_value = split2[element_index + 1]
                confidence_value = float("{0:.2f}".format(float(confidence_value)))
                taxon_rank_blca = taxon_blca.split(':')[0]

                taxon_rank_gtdb = ''
                if taxon_rank_blca == 'superkingdom':
                    taxon_rank_gtdb = 'd'
                elif taxon_rank_blca == 'phylum':
                    taxon_rank_gtdb = 'p'
                elif taxon_rank_blca == 'class':
                    taxon_rank_gtdb = 'c'
                elif taxon_rank_blca == 'order':
                    taxon_rank_gtdb = 'o'
                elif taxon_rank_blca == 'family':
                    taxon_rank_gtdb = 'f'
                elif taxon_rank_blca == 'genus':
                    taxon_rank_gtdb = 'g'
                elif taxon_rank_blca == 'species':
                    taxon_rank_gtdb = 's'

                taxon_name_blca  = ':'.join(taxon_blca.split(':')[1:]).replace(':', '')
                current_rank_with_num = '%s__%s(%s)' % (taxon_rank_gtdb, taxon_name_blca, confidence_value)
                current_rank_no_num   = '%s__%s' % (taxon_rank_gtdb, taxon_name_blca)
                formatted_taxon_list_with_num2.append(current_rank_with_num)
                formatted_taxon_list_no_num2.append(current_rank_no_num)
                element_index += 2

            formatted_taxon_str_with_num = ';'.join(formatted_taxon_list_with_num2)
            formatted_taxon_str_no_num = ';'.join(formatted_taxon_list_no_num2)

        formatted_taxon_str_with_numno_space = '_'.join(formatted_taxon_str_with_num.split(' '))
        formatted_taxon_str_no_num_no_space = '_'.join(formatted_taxon_str_no_num.split(' '))

        taxon_dict_formatted_with_num[each_16s] = formatted_taxon_str_with_numno_space
        taxon_dict_formatted_no_num[each_16s] = formatted_taxon_str_no_num_no_space

    output_file_1_handle = open(output_file_1, 'w')
    output_file_2_handle = open(output_file_2, 'w')
    for each_seq in taxon_dict_formatted_with_num:
        output_file_1_handle.write('%s\t%s\n' % (each_seq, taxon_dict_formatted_with_num[each_seq]))
        output_file_2_handle.write('%s\t%s\n' % (each_seq, taxon_dict_formatted_no_num[each_seq]))
    output_file_1_handle.close()
    output_file_2_handle.close()


def split_fasta(fasta_in, per_file_seq_num, num_of_file, output_dir):

    fasta_in_name, fasta_in_path, fasta_in_basename, fasta_in_ext = sep_path_basename_ext(fasta_in)

    if (per_file_seq_num is None) and (num_of_file is None):
        print('Please either use the "-ns" flag to define the number of sequence per file or the "-nf" flag to specify the number of files to be generated.')
        exit()
    elif (per_file_seq_num is not None) and (num_of_file is not None):
        print('Options "-ns" and "-nf" are incompatible, only specify one of them.')
        exit()
    elif (per_file_seq_num is not None) and (num_of_file is None):

        if os.path.isdir(output_dir) is True:
            print('Output folder detected, please provide a different name!')
            exit()
        else:
            os.mkdir(output_dir)

        n = 1
        for seq_record in SeqIO.parse(fasta_in, 'fasta'):
            file_index = (n - 1) // per_file_seq_num + 1
            pwd_sub_file = '%s/%s_%s.fa' % (output_dir, fasta_in_basename, file_index)
            if per_file_seq_num == 1:
                pwd_sub_file = '%s/%s.fa' % (output_dir, seq_record.id)
            # write out sequence
            with open(pwd_sub_file, 'a') as pwd_sub_file_handle:
                pwd_sub_file_handle.write('>%s\n' % seq_record.id)
                pwd_sub_file_handle.write('%s\n'  % str(seq_record.seq))
            n += 1
    elif (per_file_seq_num is None) and (num_of_file is not None):

        if os.path.isdir(output_dir) is True:
            print('Output folder detected, please provide a different name!')
            exit()
        else:
            os.mkdir(output_dir)

        # get total number of sequences
        total_seq_num = 0
        for each_seq in SeqIO.parse(fasta_in, 'fasta'):
            total_seq_num += 1
        seq_num_per_file = math.ceil(total_seq_num/num_of_file)

        # write out
        seq_index = 1
        for seq_record in SeqIO.parse(fasta_in, 'fasta'):
            file_index = math.ceil(seq_index/seq_num_per_file)
            pwd_sub_file = '%s/%s_%s.fa' % (output_dir, fasta_in_basename, file_index)

            # write out sequence
            with open(pwd_sub_file, 'a') as pwd_sub_file_handle:
                pwd_sub_file_handle.write('>%s\n' % seq_record.id)
                pwd_sub_file_handle.write('%s\n'  % str(seq_record.seq))
            seq_index += 1

def blca(fa_dir, fa_ext, output_folder, ref_seq, ref_tax, num_threads):

    pwd_current_file  = os.path.realpath(__file__)
    current_file_path = '/'.join(pwd_current_file.split('/')[:-1])
    blca_main_py      = '%s/blca_main.py' % (current_file_path)

    seq_file_re = '%s/*.%s' % (fa_dir, fa_ext)
    seq_file_list = glob.glob(seq_file_re)

    # prepare blca commands
    blca_cmd_list = []
    for seq_file in seq_file_list:
        blca_cmd = 'python3 %s -q %s -r %s -p %s -i %s -o %s' % (blca_main_py, ref_seq, ref_tax, 1, seq_file, output_folder)
        blca_cmd_list.append(blca_cmd)

    # run blca with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(os.system, blca_cmd_list)
    pool.close()
    pool.join()

    # parse blca output
    blca_op_re = '%s/*.blca.out' % output_folder
    blca_op_list = glob.glob(blca_op_re)
    for blca_op in blca_op_list:
        parse_blca_op(blca_op)


def Usearch16S(args):

    clean_data_dir      = args['i']
    clean_data_ext      = args['x']
    op_dir              = args['o']
    run_blca            = args['blca']
    ref_seq             = args['r']
    ref_tax             = args['c']
    num_threads         = args['t']
    force_create_op_dir = args['f']
    len_range_str       = args['l']

    if len_range_str.startswith('-'):
        min_len = 0
        max_len = int(len_range_str[1:])
    elif len_range_str.endswith('-'):
        min_len = int(len_range_str[:-1])
        max_len = 9999999
    else:
        min_len = int(len_range_str.split('-')[0])
        max_len = int(len_range_str.split('-')[1])

    if run_blca is True:
        check_executables(['blastn', 'blastdbcmd', 'clustalo', 'muscle', 'usearch'])

    # define file name
    cmd_txt                     = '%s/cmds.txt'                                 % op_dir
    dir_clean_data_renamed      = '%s/s01_CleanData_renamed_%s_%sbp'            % (op_dir, min_len, max_len)
    dir_DereplicatedData        = '%s/s02_DereplicatedData'                     % op_dir
    unoise_nc_fa                = '%s/s06_AllSamples_unoise_nc.fasta'           % op_dir
    unoise_nc_fa_split_dir      = '%s/s06_AllSamples_unoise_nc_split'           % op_dir
    s07_OtuTable                = '%s/s07_AllSamples_unoise_otu_table.txt'      % op_dir
    s09_FinalOtuTable           = '%s/s09_AllSamples_unoise_otu_tax.txt'        % op_dir
    s10_blca_op_dir             = '%s/s10_AllSamples_BLCA_classifications'      % op_dir
    combined_blca_blastn_tmp    = '%s/s08_AllSamples_unoise_nc.blca.blastn.tmp' % op_dir
    combined_blca_blastn_sorted = '%s/s08_AllSamples_unoise_nc.blca.blastn'     % op_dir
    combined_blca_out_tmp       = '%s/s08_AllSamples_unoise_nc.blca.out.tmp'    % op_dir
    combined_blca_out_sorted    = '%s/s08_AllSamples_unoise_nc.blca.out'        % op_dir
    combined_blca_1_txt_tmp     = '%s/s08_AllSamples_unoise_nc.blca.1.txt.tmp'  % op_dir
    combined_blca_1_txt_sorted  = '%s/s08_AllSamples_unoise_nc.blca.1.txt'      % op_dir
    combined_blca_2_txt_tmp     = '%s/s08_AllSamples_unoise_nc.blca.2.txt.tmp'  % op_dir
    combined_blca_2_txt_sorted  = '%s/s08_AllSamples_unoise_nc.blca.2.txt'      % op_dir

    # create output folder
    if os.path.isdir(op_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s' % dir_clean_data_renamed)
    os.system('mkdir %s' % dir_DereplicatedData)

    clean_data_re = '%s/*.%s' % (clean_data_dir, clean_data_ext)
    clean_data_list = glob.glob(clean_data_re)
    if len(clean_data_list) == 0:
        print('Input sequences not found, program exited!')
        exit()

    file_index = 1
    for each_file in sorted(clean_data_list):

        fa_name, _, fa_base, _  = sep_path_basename_ext(each_file)
        pwd_fa_renamed          = '%s/%s'                           % (dir_clean_data_renamed, fa_name)
        pwd_fa_uniques          = '%s/%s_uniques.fasta'             % (dir_DereplicatedData, fa_base)

        print('Running usearch -fastx_uniques on %s/%s: %s' % (file_index, len(clean_data_list), fa_base))
        file_index += 1

        # add sample id to contig id
        pwd_fa_renamed_handle = open(pwd_fa_renamed, 'w')
        for each_seq in SeqIO.parse(each_file, 'fasta'):
            seq_id = each_seq.id
            seq_len = len(each_seq.seq)
            if max_len >= seq_len >= min_len:
                pwd_fa_renamed_handle.write('>%s;sample=%s;\n' % (seq_id, fa_base))
                pwd_fa_renamed_handle.write(str(each_seq.seq) + '\n')
        pwd_fa_renamed_handle.close()

        # run dereplicate
        cmd_fastx_uniques = 'usearch -fastx_uniques %s -fastaout %s -sizeout &>/dev/null' % (pwd_fa_renamed, pwd_fa_uniques)
        with open(cmd_txt, 'a') as cmd_txt_handle:
            cmd_txt_handle.write(cmd_fastx_uniques + '\n')
        os.system(cmd_fastx_uniques)

    # combine sequences from all samples
    cmd_2 = 'cat %s/*.fasta > %s/s03_AllSamples.fasta' % (dir_DereplicatedData, op_dir)
    with open(cmd_txt, 'a') as cmd_txt_handle:
        cmd_txt_handle.write(cmd_2 + '\n')
    print(cmd_2)
    os.system(cmd_2)

    # dereplicate AllSamples.fasta
    cmd_3  = 'usearch -fastx_uniques %s/s03_AllSamples.fasta -fastaout %s/s04_AllSamples_uniques.fasta -sizein -sizeout -strand both &>/dev/null' % (op_dir, op_dir)
    with open(cmd_txt, 'a') as cmd_txt_handle:
        cmd_txt_handle.write(cmd_3 + '\n')
    print(cmd_3)
    os.system(cmd_3)

    # Generating unique sequences using UNOISE
    cmd_4  = 'usearch -unoise3 %s/s04_AllSamples_uniques.fasta -zotus %s/s05_AllSamples_denoised.fasta &>/dev/null' % (op_dir, op_dir)
    with open(cmd_txt, 'a') as cmd_txt_handle:
        cmd_txt_handle.write(cmd_4 + '\n')
    print(cmd_4)
    os.system(cmd_4)

    # Chimera Removal
    cmd_5  = 'usearch -uchime2_ref %s/s05_AllSamples_denoised.fasta -db %s -strand plus -mode high_confidence -notmatched %s/s06_AllSamples_unoise_nc.fasta &>/dev/null' % (op_dir, ref_seq, op_dir)
    if run_blca is True:
        cmd_5  = 'usearch -uchime2_ref %s/s05_AllSamples_denoised.fasta -db %s -strand plus -mode high_confidence -notmatched %s/s06_AllSamples_unoise_nc.fasta &>/dev/null' % (op_dir, ref_seq, op_dir)
    with open(cmd_txt, 'a') as cmd_txt_handle:
        cmd_txt_handle.write(cmd_5 + '\n')
    print(cmd_5)
    os.system(cmd_5)

    # generate OTU table, an OTU table is made by the otutab command
    cmd_6 = 'usearch -otutab %s/s03_AllSamples.fasta -db %s/s06_AllSamples_unoise_nc.fasta -id 0.97 -otutabout %s/s07_AllSamples_unoise_otu_table.txt &>/dev/null' % (op_dir, op_dir, op_dir)
    with open(cmd_txt, 'a') as cmd_txt_handle:
        cmd_txt_handle.write(cmd_6 + '\n')
    print(cmd_6)
    os.system(cmd_6)
    print('usearch -otutab finished')

    # Mapping of OTUs on Reference Database
    if run_blca is False:
        cmd_7  = 'blastn -query %s/s06_AllSamples_unoise_nc.fasta -outfmt 6 -out %s/s08_AllSamples_unoise_nc.txt -db %s -evalue 1e-20 -num_threads %s' % (op_dir, op_dir, ref_seq, num_threads)
        with open(cmd_txt, 'a') as cmd_txt_handle:
            cmd_txt_handle.write(cmd_7 + '\n')
        print(cmd_7)
        os.system(cmd_7)

        # keep best hit
        blast_op          = '%s/s08_AllSamples_unoise_nc_blastn.txt'          % op_dir
        blast_op_best_hit = '%s/s08_AllSamples_unoise_nc_blastn_best_hit.txt' % op_dir
        best_hit(blast_op, blast_op_best_hit)

        # Merge Table and Taxonomy
        silva_ref_to_tax_dict = dict()
        for each_seq in SeqIO.parse(ref_seq, 'fasta'):
            seq_id = each_seq.id
            seq_tax = ' '.join(each_seq.description.split(' ')[1:])
            silva_ref_to_tax_dict[seq_id] = seq_tax

        otu_best_hit_dict = dict()
        for each in open(blast_op_best_hit):
            each_split = each.strip().split('\t')
            info_list = [each_split[1], each_split[2], each_split[3], each_split[10]]
            otu_best_hit_dict[each_split[0]] = info_list

        s09_FinalOtuTable_handle = open(s09_FinalOtuTable, 'w')
        for each_line in open(s07_OtuTable):
            if each_line.startswith('#'):
                s09_FinalOtuTable_handle.write('ID\tIdentity\tAlignment_Length\tEvalue\tTaxonomy\n')
            else:
                each_line_split = each_line.strip().split('\t')
                otu_id = each_line_split[0]
                best_hit_info_list = otu_best_hit_dict.get(otu_id, ['na', 'na', 'na', 'na'])
                best_hit_info_list_without_ref_id = []
                best_hit_info_list_without_ref_id.append(best_hit_info_list[1])
                best_hit_info_list_without_ref_id.append(best_hit_info_list[2])
                best_hit_info_list_without_ref_id.append(best_hit_info_list[3])
                best_hit_info_str = '\t'.join(best_hit_info_list_without_ref_id)
                ref_id = best_hit_info_list[0]
                ref_tax = silva_ref_to_tax_dict.get(ref_id, 'na')
                s09_FinalOtuTable_handle.write('%s\t%s\t%s\n' % (otu_id, best_hit_info_str, ref_tax))
        s09_FinalOtuTable_handle.close()

    else:
        # split fasta file
        print('split fasta')
        split_fasta(unoise_nc_fa, None, num_threads, unoise_nc_fa_split_dir)

        # create blca output directory
        os.system('mkdir %s' % s10_blca_op_dir)

        # run blca
        print('running blca')
        blca(unoise_nc_fa_split_dir, 'fa', s10_blca_op_dir, ref_seq, ref_tax, num_threads)

        # combine results
        os.system('cat %s/*.blastn > %s'       % (s10_blca_op_dir, combined_blca_blastn_tmp))
        os.system('cat %s/*.blca.out > %s'     % (s10_blca_op_dir, combined_blca_out_tmp))
        os.system('cat %s/*.blca.1.txt > %s'   % (s10_blca_op_dir, combined_blca_1_txt_tmp))
        os.system('cat %s/*.blca.2.txt > %s'   % (s10_blca_op_dir, combined_blca_2_txt_tmp))

        # sort results
        os.system('cat %s | sort > %s' % (combined_blca_blastn_tmp, combined_blca_blastn_sorted))
        os.system('cat %s | sort > %s' % (combined_blca_out_tmp, combined_blca_out_sorted))
        os.system('cat %s | sort > %s' % (combined_blca_1_txt_tmp, combined_blca_1_txt_sorted))
        os.system('cat %s | sort > %s' % (combined_blca_2_txt_tmp, combined_blca_2_txt_sorted))

        # remove tmp files
        os.system('rm -r %s' % s10_blca_op_dir)
        os.system('rm %s'    % combined_blca_blastn_tmp)
        os.system('rm %s'    % combined_blca_out_tmp)
        os.system('rm %s'    % combined_blca_1_txt_tmp)
        os.system('rm %s'    % combined_blca_2_txt_tmp)

    # final report
    print('OTU table exported to: %s' % s07_OtuTable)
    if run_blca is False:
        print('OTU taxonomy exported to: %s' % s09_FinalOtuTable)
    else:
        print('OTU taxonomy exported to: %s and %s' % (combined_blca_1_txt_sorted, combined_blca_2_txt_sorted))
    print('Done')


if __name__ == '__main__':

    Usearch16S_parser = argparse.ArgumentParser(usage=Usearch16S_usage)
    Usearch16S_parser.add_argument('-i',       required=True,                        help='path to input sequences')
    Usearch16S_parser.add_argument('-x',       required=True,                        help='file extension')
    Usearch16S_parser.add_argument('-o',       required=True,                        help='output directory')
    Usearch16S_parser.add_argument('-l',       required=False,                       help='allowed length range, default is 0-9999999')
    Usearch16S_parser.add_argument('-r',       required=False,                       help='reference sequences')
    Usearch16S_parser.add_argument('-blca',    required=False, action="store_true",  help='perform classification with BLCA')
    Usearch16S_parser.add_argument('-c',       required=False,                       help='reference sequence taxonomy')
    Usearch16S_parser.add_argument('-t',       required=False, type=int, default=1,  help='number of threads, default is 1')
    Usearch16S_parser.add_argument('-f',       required=False, action="store_true",  help='force overwrite')
    args = vars(Usearch16S_parser.parse_args())
    Usearch16S(args)



import os
import glob
from Bio import SeqIO


file_dir    = 'AOA_2279_dRep99_1679_for_mapping_renamed'
file_ext    = 'fna'
op_dir      = 'AOA_2279_dRep99_1679_for_mapping_renamed_2kbp'


file_re = '%s/*.%s' % (file_dir, file_ext)
file_list = glob.glob(file_re)

for each_file in file_list:
    file_name = os.path.basename(each_file)
    op_file = '%s/%s' % (op_dir, file_name)
    op_file_handle = open(op_file, 'w')
    for each_seq in SeqIO.parse(each_file, 'fasta'):
        if len(each_seq.seq) >= 2000:
            op_file_handle.write('>%s\n' % each_seq.id)
            op_file_handle.write('%s\n'  % each_seq.seq)
        else:
            print(each_seq.id)
    op_file_handle.close()



import argparse


statsTaxa_usage = '''
============ statsTaxa example commands ============

BioSAK statsTaxa -i taxa.txt -o taxa_stats.txt

# Example of input file (no header, GTDB format):
d__Archaea;p__Thermoproteota;c__Nitrososphaeria;o__Nitrososphaerales;f__Nitrosopumilaceae;g__JAHRAL01;s__JAHRAL01 sp003724275
d__Archaea;p__Thermoproteota;c__Nitrososphaeria;o__Nitrososphaerales;f__Nitrosopumilaceae;g__WTHC01;s__WTHC01 sp026708305
d__Archaea;p__Thermoproteota;c__Nitrososphaeria;o__Nitrososphaerales;f__Nitrosopumilaceae;g__TA-20;s__TA-20 sp013287585
d__Archaea;p__Thermoproteota;c__Nitrososphaeria;o__Nitrososphaerales;f__Nitrosopumilaceae;g__Nitrosopumilus;s__

====================================================
'''


def statsTaxa(args):

    taxa_txt        = args['i']
    taxa_stats_txt  = args['o']

    tax_rank_order_list = []
    stats_dod = dict()
    for each_line in open(taxa_txt):
        each_line_split = each_line.strip().split(';')
        for each_r in each_line_split:
            tax_rank = each_r.split('__')[0]

            if tax_rank not in tax_rank_order_list:
                tax_rank_order_list.append(tax_rank)

            if tax_rank not in stats_dod:
                stats_dod[tax_rank] = dict()

            if each_r not in stats_dod[tax_rank]:
                stats_dod[tax_rank][each_r] = 1
            else:
                stats_dod[tax_rank][each_r] += 1

    taxa_stats_txt_handle = open(taxa_stats_txt, 'w')
    for each_rank in tax_rank_order_list:
        current_rank_stats_dict = stats_dod[each_rank]
        tax_sorted = [i[0] for i in sorted(current_rank_stats_dict.items(), key=lambda x: x[1])][::-1]
        for each_tax in tax_sorted:
            taxa_stats_txt_handle.write('%s\t%s\t%s\n' % (each_rank, each_tax, current_rank_stats_dict[each_tax]))
    taxa_stats_txt_handle.close()

    print('Done!')


if __name__ == '__main__':

    statsTaxa_parser = argparse.ArgumentParser(usage=statsTaxa_usage)
    statsTaxa_parser.add_argument('-i', required=True, help='input txt')
    statsTaxa_parser.add_argument('-o', required=True, help='output txt')
    args = vars(statsTaxa_parser.parse_args())
    statsTaxa(args)



import argparse


metadata_usage = '''
======================================= metadata example commands =======================================

BioSAK metadata -id_col 1 -i meta.txt -o meta_new.txt -a checkm_quality.tsv -c 'Completeness,Contamination'
BioSAK metadata -id_col 2 -i meta.txt -o meta_new.txt -a gtdb_taxonomy.tsv -c classification

=========================================================================================================
'''


def metadata(args):

    metadata_txt_in       = args['i']
    new_attribute_file    = args['a']
    interested_col_header = args['c']
    metadata_txt_out      = args['o']
    metadata_txt_sep      = args['ms']
    new_attribute_sep     = args['as']
    id_column             = args['id_col']

    interested_col_list = interested_col_header.split(',')
    default_key_str = metadata_txt_sep.join(['na']*len(interested_col_list))

    # read in new attribute
    new_attribute_dict = dict()
    col_index = dict()
    line_num_index = 0
    for each_line in open(new_attribute_file):
        line_split = each_line.strip().split(new_attribute_sep)
        if line_num_index == 0:
            col_index = {key: i for i, key in enumerate(line_split)}
        else:
            sample_id = line_split[0]
            value_list = []
            for interested_col in interested_col_list:
                value_list.append(line_split[col_index[interested_col]])
            value_str = metadata_txt_sep.join(value_list)
            new_attribute_dict[sample_id] = value_str
        line_num_index += 1

    # add new attribute to metadata file
    metadata_txt_out_handle = open(metadata_txt_out, 'w')
    line_num_index = 0
    for each_line in open(metadata_txt_in):
        each_line_split = each_line.strip().split(metadata_txt_sep)
        sample_id = each_line_split[id_column - 1]
        if line_num_index == 0:
            metadata_txt_out_handle.write('%s%s%s\n' % (each_line.strip(), metadata_txt_sep, metadata_txt_sep.join(interested_col_list)))
        else:
            metadata_txt_out_handle.write('%s%s%s\n' % (each_line.strip(), metadata_txt_sep, new_attribute_dict.get(sample_id, default_key_str)))
        line_num_index += 1
    metadata_txt_out_handle.close()


if __name__ == '__main__':

    metadata_parser = argparse.ArgumentParser(usage=metadata_usage)
    metadata_parser.add_argument('-i',  required=True,                   help='input metadata file')
    metadata_parser.add_argument('-o',  required=True,                   help='output metadata file')
    metadata_parser.add_argument('-a',  required=True,                   help='new attribute file')
    metadata_parser.add_argument('-c',  required=True,                   help='interested column header')
    metadata_parser.add_argument('-ms', required=False, default='\t',    help='metadata column separator')
    metadata_parser.add_argument('-as', required=False, default='\t',    help='new attribute file column separator')
    metadata_parser.add_argument('-id_col', required=False, type=int, default=1,    help='id column in input metadata file, default is 1 (the first column)')
    args = vars(metadata_parser.parse_args())
    metadata(args)



import argparse


usearch_uc_usage = '''
=================== usearch_uc example commands ===================

BioSAK usearch_uc -uc uclust_0.6.uc -o uclust_0.6_min1.txt
BioSAK usearch_uc -uc uclust_0.6.uc -o uclust_0.6_min3.txt -n 3

===================================================================
'''


def usearch_uc(args):

    uc_file     = args['uc']
    min_seq_num = args['n']
    output_file = args['o']

    cluster_id_set = set()
    cluster_to_seq_member_dict = {}
    for each_line in open(uc_file):
        each_line_split = each_line.strip().split('\t')
        cluster_id = each_line_split[1]
        seq_id = each_line_split[8].split(' ')[0]
        cluster_id_set.add(int(cluster_id))

        if cluster_id not in cluster_to_seq_member_dict:
            cluster_to_seq_member_dict[cluster_id] = {seq_id}
        else:
            cluster_to_seq_member_dict[cluster_id].add(seq_id)

    # write out cluster sequence members
    qualified_cluster_num = 0
    output_file_handle = open(output_file, 'w')
    for each_cluster in sorted([i for i in cluster_id_set]):
        current_cluster_seqs = cluster_to_seq_member_dict[str(each_cluster)]
        current_cluster_seqs_sorted = sorted([i for i in current_cluster_seqs])

        if len(current_cluster_seqs) >= min_seq_num:
            output_file_handle.write('Cluster_%s\t%s\n' % (each_cluster, ','.join(current_cluster_seqs_sorted)))
            qualified_cluster_num += 1
    output_file_handle.close()

    if min_seq_num == 1:
        print('Found %s clusters with at least %s sequence.' % (qualified_cluster_num, min_seq_num))
    if min_seq_num > 1:
        print('Found %s clusters with at least %s sequences.' % (qualified_cluster_num, min_seq_num))


if __name__ == '__main__':

    usearch_uc_parser = argparse.ArgumentParser(usage=usearch_uc_usage)
    usearch_uc_parser.add_argument('-uc', required=True,                        help='uc file from Usearch')
    usearch_uc_parser.add_argument('-n',  required=False, type=int, default=1,  help='minimum number of sequence in a cluster, default: 1')
    usearch_uc_parser.add_argument('-o',  required=True,                        help='output file')
    args = vars(usearch_uc_parser.parse_args())
    usearch_uc(args)




import os
import argparse


RunGraphMB_usage = '''
=============== RunGraphMB example commands ===============

BioSAK RunGraphMB -gfa assembly_graph.gfa -o output_dir

===========================================================
'''


def gfa_to_depth(gfa_file, edge_fasta, depth_txt, depth_txt_MetaBAT):

    # parse gfa file
    edge_id_set = set()
    edge_seq_dict = dict()
    edge_len_dict = dict()
    edge_depth_dict = dict()
    for each_line in open(gfa_file):
        each_line_split = each_line.strip().split('\t')
        if each_line.startswith('S'):
            edge_id = each_line_split[1]
            edge_seq = each_line_split[2]
            edge_len = len(edge_seq)

            try:
                edge_depth = int(each_line_split[3].split(':')[-1])
            except:
                edge_depth = float(each_line_split[3].split(':')[-1])

            edge_id_set.add(edge_id)
            edge_len_dict[edge_id] = edge_len
            edge_seq_dict[edge_id] = edge_seq
            edge_depth_dict[edge_id] = edge_depth

    # write out edge sequence
    edge_fasta_handle = open(edge_fasta, 'w')
    for each_edge in sorted([i for i in edge_id_set]):
        edge_seq = edge_seq_dict[each_edge]
        edge_fasta_handle.write('>%s\n' % each_edge)
        edge_fasta_handle.write('%s\n' % edge_seq)
    edge_fasta_handle.close()

    # write out depth info
    depth_txt_handle = open(depth_txt, 'w')
    depth_txt_handle.write('Sequence\tLength(bp)\tDepth(x)\n')
    for each_edge in sorted([i for i in edge_id_set]):
        edge_len = edge_len_dict[each_edge]
        edge_depth = edge_depth_dict[each_edge]
        depth_txt_handle.write('%s\t%s\t%s\n' % (each_edge, edge_len, edge_depth))
    depth_txt_handle.close()

    # write out depth info (MetaBAT format)
    depth_txt_MetaBAT_handle = open(depth_txt_MetaBAT, 'w')
    depth_txt_MetaBAT_handle.write('contigName\tcontigLen\ttotalAvgDepth\tsample1\tsample1-var\n')
    for each_edge in sorted([i for i in edge_id_set]):
        edge_len = edge_len_dict[each_edge]
        edge_depth = edge_depth_dict[each_edge]
        fake_depth_var = float("{0:.2f}".format(edge_depth*0.01))
        depth_txt_MetaBAT_handle.write('%s\t%s\t%s\t%s\t%s\n' % (each_edge, edge_len, edge_depth, edge_depth, fake_depth_var))
    depth_txt_MetaBAT_handle.close()


def RunGraphMB(arg_dict):

    gfa_file = arg_dict['gfa']
    op_dir   = arg_dict['o']

    # create folder
    if os.path.isdir(op_dir):
        os.system('rm -r %s' % op_dir)
    os.system('mkdir %s' % op_dir)

    # 1. copy gfa_file into GraphMB_input_dir
    os.system('cp %s %s/assembly_graph.gfa' % (gfa_file, op_dir))

    # 2. prepare depth file and edge sequence file
    depth_file_tmp = '%s/assembly_depth.tmp.txt'    % op_dir
    depth_file     = '%s/assembly_depth.txt'        % op_dir
    sequence_file  = '%s/assembly.fasta'            % op_dir
    gfa_to_depth(gfa_file, sequence_file, depth_file_tmp, depth_file)
    #os.system('rm %s' % depth_file_tmp)

    # 3. command for running GraphMB
    print('# Example commands for running GraphMB')
    cmd_GraphMB = 'conda activate graphmb\ngraphmb --assembly %s --outdir GraphMB_output_dir' % op_dir
    print(cmd_GraphMB)


if __name__ == '__main__':

    RunGraphMB_parser = argparse.ArgumentParser(usage=RunGraphMB_usage)
    RunGraphMB_parser.add_argument('-gfa', required=True, help='gfa file')
    RunGraphMB_parser.add_argument('-o',   required=True, help='output folder (i.e., input folder to GraphMB)')
    args = vars(RunGraphMB_parser.parse_args())
    RunGraphMB(args)



import os
import glob
import argparse
from Bio import SeqIO


mean_MAG_cov_usage = '''
==================== mean_MAG_cov example commands ====================

# Example command
BioSAK mean_MAG_cov -d ctg_lt2500_depth.txt -b bin_folder -x fasta

Format of contig depth file: tab separated, no header!!!
ctg_1   8
ctg_2   26.1

=======================================================================
'''


def mean_MAG_cov(args):

    # read in arguments
    metabat_depth_file = args['d']
    bin_folder         = args['b']
    bin_file_extension = args['x']

    # check input file
    if bin_folder[-1] == '/':
      bin_folder = bin_folder[:-1]

    # get bin file list
    bin_file_re = '%s/*.%s' % (bin_folder, bin_file_extension)
    bin_file_list = [os.path.basename(file_name) for file_name in glob.glob(bin_file_re)]

    # check whether input bin files detected
    if len(bin_file_list) == 0:
        print('No input file detected, program exit!')
        exit()

    # read in depth info
    ctg_depth_dict = {}
    for each_line in open(metabat_depth_file):
        each_line_split = each_line.strip().split('\t')
        ctg_id = each_line_split[0]
        ctg_depth = float(each_line_split[1])
        ctg_depth_dict[ctg_id] = ctg_depth

    file_re = '%s/*.%s' % (bin_folder, bin_file_extension)
    file_list = [os.path.basename(file_name) for file_name in glob.glob(file_re)]

    gnm_mean_depth_dict = {}
    print('MAG\tLength(bp)\tDepth')
    for genome in file_list:
        pwd_genome = '%s/%s' % (bin_folder, genome)
        gnm_total_len = 0
        gnm_total_depth = 0
        for each_ctg in SeqIO.parse(pwd_genome, 'fasta'):
            gnm_total_len += len(each_ctg.seq)
            gnm_total_depth += (len(each_ctg.seq) * ctg_depth_dict[each_ctg.id])
        gnm_mean_depth = gnm_total_depth / gnm_total_len
        gnm_mean_depth = float("{0:.2f}".format(gnm_mean_depth))
        print('%s\t%s\t%s' % (genome, gnm_total_len, gnm_mean_depth))
        gnm_mean_depth_dict[genome] = gnm_mean_depth


if __name__ == "__main__":
    mean_MAG_cov_parser = argparse.ArgumentParser()
    mean_MAG_cov_parser.add_argument('-d', required=True, help='MetaBAT produced depth file')
    mean_MAG_cov_parser.add_argument('-b', required=True, help='bin folder')
    mean_MAG_cov_parser.add_argument('-x', required=True, help='file extension')
    args = vars(mean_MAG_cov_parser.parse_args())
    mean_MAG_cov(args)




import argparse
from Bio import SeqIO
from BioSAK.global_functions import sep_path_basename_ext


GTDB_for_BLCA_usage = '''
===== GTDB_for_BLCA example commands =====

BioSAK GTDB_for_BLCA -i ssu_all_r220.fna

# output_file:
ssu_all_r220.blca.fa
ssu_all_r220.blca.tax

==========================================
'''


def GTDB_for_BLCA(args):

    GTDB_db_file = args['i']

    db_path, db_base, db_ext = sep_path_basename_ext(GTDB_db_file)
    file_out_sequence = '%s/%s.blca.fa'     % (db_path, db_base)
    file_out_taxonomy = '%s/%s.blca.tax'    % (db_path, db_base)

    rank_list = ['species', 'genus', 'family', 'order', 'class', 'phylum', 'superkingdom']

    file_out_sequence_handle = open(file_out_sequence, 'w')
    file_out_taxonomy_handle = open(file_out_taxonomy, 'w')
    for seq_record in SeqIO.parse(GTDB_db_file, 'fasta'):
        seq_record_taxon_split = ' '.join(seq_record.description.strip().split(' [')[0].split(' ')[1:]).split(';')
        seq_record_taxon_split_no_rank = [i[3:] for i in seq_record_taxon_split]
        seq_record_taxon_split_no_rank_reverse = seq_record_taxon_split_no_rank[::-1]

        GTDB_seq_taxon_str = ''
        n = 0
        for taxon_rank in rank_list:
            GTDB_seq_taxon_str += '%s:%s;' % (taxon_rank, seq_record_taxon_split_no_rank_reverse[n])
            n += 1

        # write out to taxonomy file
        file_out_taxonomy_handle.write('%s\t%s\n' % (seq_record.id, GTDB_seq_taxon_str))

        # write out to sequence file
        file_out_sequence_handle.write('>%s\n' % seq_record.description)
        file_out_sequence_handle.write('%s\n' % seq_record.seq)

    file_out_sequence_handle.close()
    file_out_taxonomy_handle.close()

    print('Done!')
    print('You may need to run:')
    print('makeblastdb -dbtype nucl -parse_seqids -in %s' % file_out_sequence)


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument('-i', required=True, help='GTDB SSU file, e.g. ssu_all_r220.fna')
    args = vars(parser.parse_args())
    GTDB_for_BLCA(args)



import os
import glob
import argparse
from Bio import SeqIO


GenBank_usage = '''
====================== GenBank example commands ======================

BioSAK GenBank -i accession.txt -o op_dir -f -tax ncbi_taxonomy.txt

-tax /Users/songweizhi/DB/taxdump_20250321/ncbi_taxonomy.txt

======================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]

    return f_name, f_path, f_base, f_ext


def get_origin_seq_from_gbk(gbk_file):

    after_origin_line = False
    before_slash_line = True
    concatenated_seq = ''
    for each_line in open(gbk_file):
        each_line = each_line.strip()
        if each_line == 'ORIGIN':
            after_origin_line = True
        if each_line == '//':
            before_slash_line = False
        if (after_origin_line is True) and (before_slash_line is True):
            if each_line != 'ORIGIN':
                each_line_split = each_line.split(' ')
                seq_str = ''.join(each_line_split[1:])
                seq_str = seq_str.upper()
                concatenated_seq += seq_str

    return concatenated_seq


def combine_esearch_op_organism(op_dir, tax_lineage_dict, op_txt):

    file_re = '%s/*_organism.txt' % op_dir
    file_list = glob.glob(file_re)

    op_txt_handle = open(op_txt, 'w')
    for each_file in file_list:
        f_name, f_path, f_base, f_ext = sep_path_basename_ext(each_file)
        accession_id = f_base.split('_organism')[0]

        organism_info = ''
        with open(each_file) as f:
            organism_info = f.readline().replace('ORGANISM', '').strip()
            organism_g = organism_info.split()[0]
            g_full_lineage = tax_lineage_dict.get(organism_g, ('g__' + organism_g))
            full_lineage_str = '%s;s__%s' % (g_full_lineage, organism_info)
        if len(tax_lineage_dict) == 0:
            op_txt_handle.write('%s\t%s\n' % (accession_id, organism_info))
        else:
            op_txt_handle.write('%s\t%s\n' % (accession_id, full_lineage_str))
    op_txt_handle.close()


def combine_esearch_op_voucher(op_dir, op_txt):

    file_re = '%s/*_voucher.txt' % op_dir
    file_list = glob.glob(file_re)

    op_txt_handle = open(op_txt, 'w')
    for each_file in file_list:
        f_name, f_path, f_base, f_ext = sep_path_basename_ext(each_file)
        accession_id = f_base.split('_voucher')[0]

        voucher_info = ''
        with open(each_file) as f:
            #voucher_info = f.readline().replace('ORGANISM', '').strip()
            voucher_info = f.readline().strip()
            if 'specimen_voucher=' in voucher_info:
                voucher_info = voucher_info.replace('/specimen_voucher="', '')[:-1]
        if voucher_info != '':
            op_txt_handle.write('%s\t%s\n' % (accession_id, voucher_info))
    op_txt_handle.close()


def GenBank(args):

    accession_txt       = args['i']
    op_dir              = args['o']
    force_overwrite     = args['f']
    tax_file            = args['tax']

    tmp_dir                     = '%s/tmp'                      % op_dir
    cmd_txt_gbk                 = '%s/cmds_get_gbk.sh'          % op_dir
    combined_fa                 = '%s/accession_sequence.fasta' % op_dir
    combined_organism_info_txt  = '%s/accession_organism.txt'   % op_dir
    combined_voucher_info_txt   = '%s/accession_voucher.txt'    % op_dir

    if os.path.isdir(op_dir):
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('%s already exists, program exited!' % op_dir)
            exit()
    os.mkdir(op_dir)
    os.mkdir(tmp_dir)

    cmd_list_get_gbk = []
    for accession_id in open(accession_txt):

        accession_id = accession_id.strip().split()[0]
        gbk_file     = '%s/%s.gbk'          % (tmp_dir, accession_id)
        fa_file      = '%s/%s.fasta'        % (tmp_dir, accession_id)
        voucher_txt  = '%s/%s_voucher.txt'  % (tmp_dir, accession_id)
        organism_txt = '%s/%s_organism.txt' % (tmp_dir, accession_id)

        get_gbk_cmd  = 'esearch -db nucleotide -query %s | efetch -format gb > %s' % (accession_id, gbk_file)
        os.system(get_gbk_cmd)
        cmd_list_get_gbk.append(get_gbk_cmd)

        # get organism and voucher info
        os.system('cat %s | grep "ORGANISM" > %s'         % (gbk_file, organism_txt))
        os.system('cat %s | grep "specimen_voucher" > %s' % (gbk_file, voucher_txt))

        # get sequence file
        gbk_record = SeqIO.read(gbk_file, "genbank")
        seq_id   = gbk_record.id
        seq_desc = gbk_record.description
        seq_str  = get_origin_seq_from_gbk(gbk_file)
        with open(fa_file, 'w') as f1:
            f1.write('>%s %s\n' % (seq_id, seq_desc))
            f1.write('%s\n' % seq_str)

    # write out command
    with open(cmd_txt_gbk, 'a') as f2:
        f2.write('\n'.join(cmd_list_get_gbk))

    ###################################### combine the results for all accessions ######################################

    tax_lineage_dict = dict()
    if tax_file is not None:
        for each_line in open(tax_file):
            each_line_split = each_line.strip().split('\t')
            tax_lineage_dict[each_line_split[0]] = each_line_split[1]

    os.system('cat %s/*.fasta > %s' % (tmp_dir, combined_fa))
    combine_esearch_op_organism(tmp_dir, tax_lineage_dict, combined_organism_info_txt)
    combine_esearch_op_voucher(tmp_dir, combined_voucher_info_txt)


if __name__ == '__main__':

    GenBank_parser = argparse.ArgumentParser(usage=GenBank_usage)
    GenBank_parser.add_argument('-i',          required=True,                       help='input txt containing accession id')
    GenBank_parser.add_argument('-o',          required=True,                       help='output dir')
    GenBank_parser.add_argument('-tax',        required=False, default=None,        help='NCBI taxonomy file, specify to get the full taxon lineage')
    GenBank_parser.add_argument('-f',          required=False, action="store_true", help='force overwrite')
    args = vars(GenBank_parser.parse_args())
    GenBank(args)



import os
import argparse


abd_16s_usage = '''
======================== abd_16s example commands ========================

BioSAK abd_16s -i 16S.fa -r1 R1.fa -r2 R2.fa -o op_dir -p demo -t 12 -f 

==========================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def abd_16s(args):

    input_16s_seq           = args['i']
    reads_r1_fasta          = args['r1']
    reads_r2_fasta          = args['r2']
    op_dir                  = args['o']
    op_prefix               = args['p']
    num_threads             = args['t']
    force_overwrite         = args['f']

    # create op dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output directory detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    ########################################################################################################################

    _, _, input_16s_base, _ = sep_path_basename_ext(input_16s_seq)

    bowtie_build_cmd        = 'bowtie2-build --quiet --threads %s -f %s %s'                                                                                                     % (num_threads, input_16s_seq, input_16s_base)
    bowtie_read_to_16s_cmd  = 'bowtie2 -x %s -U %s,%s -S %s/%s.sam -p %s -f --xeq --local --all --no-unal -N 1 -L 30 --quiet'                                                   % (input_16s_base, reads_r1_fasta, reads_r2_fasta, op_dir, op_prefix, num_threads)
    samtools_view_cmd       = 'samtools view -@ 32 -bS -h -b %s/%s.sam > %s/%s.bam'                                                                                             % (op_dir, op_prefix, op_dir, op_prefix)
    samtools_sort_cmd       = 'samtools sort -@ 32 %s/%s.bam -o %s/%s.sorted.bam'                                                                                               % (op_dir, op_prefix, op_dir, op_prefix)
    coverm_filter_cmd       = 'coverm filter -b %s/%s.sorted.bam --min-read-aligned-percent 0.9 --min-read-percent-identity 0.99 --output-bam-files %s/%s.sorted_filtered.bam'  % (op_dir, op_prefix, op_dir, op_prefix)
    pileup_sh_cmd           = 'pileup.sh in=%s/%s.sorted_filtered.bam out=%s/%s.sorted_filtered.cov rpkm=%s/%s.sorted_filtered.rpkm overwrite=true'                             % (op_dir, op_prefix, op_dir, op_prefix, op_dir, op_prefix)
    seqkit_stat_cmd         = 'seqkit stat %s > %s/%s.stat'                                                                                                                     % (reads_r1_fasta, op_dir, op_prefix)

    os.system(bowtie_build_cmd)
    os.system(bowtie_read_to_16s_cmd)
    os.system(samtools_view_cmd)
    os.system(samtools_sort_cmd)
    os.system(coverm_filter_cmd)
    os.system(pileup_sh_cmd)
    os.system(seqkit_stat_cmd)


if __name__ == '__main__':

    abd_16s_parser = argparse.ArgumentParser()
    abd_16s_parser.add_argument('-i',   required=True,                          help='16S sequences')
    abd_16s_parser.add_argument('-r1',  required=True,                          help='reads r1')
    abd_16s_parser.add_argument('-r2',  required=True,                          help='reads r2')
    abd_16s_parser.add_argument('-o',   required=True,                          help='output directory')
    abd_16s_parser.add_argument('-p',   required=True,                          help='output prefix')
    abd_16s_parser.add_argument('-t',   required=False, type=int, default=1,    help='number of threads, default is 1')
    abd_16s_parser.add_argument('-f',   required=False, action="store_true",    help='force overwrite')
    args = vars(abd_16s_parser.parse_args())
    abd_16s(args)



import os
import argparse


metabat2concoct_usage = '''
================= metabat2concoct example commands =================

BioSAK metabat2concoct -i metabat_depth.txt -o concoct_depth.txt

====================================================================
'''

def metabat2concoct(args):

    metabat_depth_txt       = args['i']
    concoct_depth_txt       = args['o']

    concoct_depth_txt_handle = open(concoct_depth_txt, 'w')
    for each_line in open(metabat_depth_txt):
        each_line_split = each_line.strip().split('\t')

        col_index = 0
        cols_to_keep = []
        for each_col in each_line_split:

            if col_index == 0:
                cols_to_keep.append(each_col)
            elif col_index >= 3:
                if (col_index % 2) != 0:
                    cols_to_keep.append(each_col)
            col_index += 1
        concoct_depth_txt_handle.write('\t'.join(cols_to_keep) + '\n')
    concoct_depth_txt_handle.close()


if __name__ == '__main__':

    metabat2concoct_parser = argparse.ArgumentParser(usage=metabat2concoct_usage)
    metabat2concoct_parser.add_argument('-i',   required=True,  help='metabat depth')
    metabat2concoct_parser.add_argument('-o',   required=True,  help='concoct depth')
    args = vars(metabat2concoct_parser.parse_args())
    metabat2concoct(args)



import os
import argparse
import pandas as pd


combine_low_abd_otu_usage = '''
================== combine_low_abd_otu example commands ==================

# keep OTUs with relative abundance higher than 50% and combine the rests
BioSAK combine_low_abd_otu -i otu_table.txt -c 0.5 -o otu_table_combine_min_50.txt

==========================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]

    return f_name, f_path, f_base, f_ext


def transpose_csv(file_in, file_out, sep_symbol, column_name_pos, row_name_pos):
    csv = pd.read_csv(file_in, sep=sep_symbol, header=column_name_pos, index_col=row_name_pos)
    df_csv = pd.DataFrame(data=csv)
    transposed_csv = df_csv.T
    transposed_csv.to_csv(file_out, sep=sep_symbol)


def combine_low_abd_otu(args):

    otu_table_in          = args['i']
    abd_cutoff            = args['c']
    otu_table_out         = args['o']
    interested_sample_txt = args['s']

    interested_sample_set = set()
    if interested_sample_txt is not None:
        if os.path.isfile(interested_sample_txt):
            for sample in open(interested_sample_txt):
                interested_sample_set.add(sample.strip().split()[0])
        else:
            print('%s not found, program exited!' % interested_sample_txt)
            exit()

    f_name, f_path, f_base, f_ext = sep_path_basename_ext(otu_table_out)

    # define tmp file name
    tmp1_t               = '%s.tmp1'        % otu_table_out
    tmp2_t_filtered      = '%s.tmp2'        % otu_table_out
    tmp2_t_filtered_norm = '%s.tmp2.norm'   % otu_table_out
    otu_table_out_norm   = '%s/%s_pct.%s'   % (f_path, f_base, f_ext)

    transpose_csv(otu_table_in, tmp1_t, '\t', 0, 0)

    # get df sample set
    df_sample_set = set()
    line_index = 0
    for each_line in open(tmp1_t):
        each_line_split = each_line.strip().split('\t')
        if line_index > 0:
            sample_id = each_line_split[0]
            df_sample_set.add(sample_id)

    if len(interested_sample_set) == 0:
        interested_sample_set = df_sample_set

    # get all OTUs that have abundance higher than specified cutoff in at least one sample
    otus_to_keep = set()
    line_index = 0
    otu_id_list = []
    for each_line in open(tmp1_t):
        each_line_split = each_line.strip().split('\t')
        if line_index == 0:
            otu_id_list = each_line_split
        else:
            sample_id = each_line_split[0]
            if sample_id in interested_sample_set:
                count_list = [int(i) for i in each_line_split[1:]]
                count_sum = sum(count_list)
                for (otu_id, otu_count) in zip(otu_id_list, count_list):
                    if (otu_count / count_sum) >= abd_cutoff:
                        otus_to_keep.add(otu_id)
        line_index += 1

    # filter
    otu_count_dict = dict()
    line_index = 0
    otu_id_list = []
    for each_line in open(tmp1_t):
        each_line_split = each_line.strip().split('\t')
        if line_index == 0:
            otu_id_list = each_line_split
        else:
            sample_id = each_line_split[0]
            if sample_id in interested_sample_set:
                otu_count_list = [int(i) for i in each_line_split[1:]]
                current_sample_otu_count_dict = dict()
                for (otu_id, otu_count) in zip(otu_id_list, otu_count_list):
                    if otu_id in otus_to_keep:
                        current_sample_otu_count_dict[otu_id] = otu_count
                    else:
                        if 'others' not in current_sample_otu_count_dict:
                            current_sample_otu_count_dict['others'] = 0
                        current_sample_otu_count_dict['others'] += otu_count
                otu_count_dict[sample_id] = current_sample_otu_count_dict
        line_index += 1

    otus_to_keep.add('others')
    otus_to_keep_list_sorted = sorted(list(otus_to_keep))

    # write out
    otu_table_txt_t_filtered_handle = open(tmp2_t_filtered, 'w')
    otu_table_txt_t_filtered_handle.write('\t%s\n' % '\t'.join(otus_to_keep_list_sorted))
    for each_sample in otu_count_dict:
        combined_otu_count_dict = otu_count_dict[each_sample]
        count_list= [each_sample]
        for eahc_otu in otus_to_keep_list_sorted:
            count_list.append(combined_otu_count_dict[eahc_otu])
        count_list = [str(i) for i in count_list]
        otu_table_txt_t_filtered_handle.write('\t'.join(count_list) + '\n')
    otu_table_txt_t_filtered_handle.close()

    tmp2_t_filtered_norm_handle = open(tmp2_t_filtered_norm, 'w')
    line_index = 0
    for each_line in open(tmp2_t_filtered):
        each_line_split = each_line.strip().split('\t')
        if line_index == 0:
            tmp2_t_filtered_norm_handle.write(each_line)
        else:
            sample_id = each_line_split[0]
            value_list = [int(i) for i in each_line_split[1:]]
            otu_sum = sum(value_list)
            value_list_norm = [float("{0:.4f}".format(i*100/otu_sum)) for i in value_list]
            tmp2_t_filtered_norm_handle.write('%s\t%s\n' % (sample_id, '\t'.join([str(i) for i in value_list_norm])))
        line_index += 1
    tmp2_t_filtered_norm_handle.close()

    transpose_csv(tmp2_t_filtered, otu_table_out, '\t', 0, 0)
    transpose_csv(tmp2_t_filtered_norm, otu_table_out_norm, '\t', 0, 0)


    # remove tmp files
    os.remove(tmp1_t)
    os.remove(tmp2_t_filtered)
    os.remove(tmp2_t_filtered_norm)
    print('Results exported to:\n%s\n%s' % (otu_table_out, otu_table_out_norm))
    print('Done!')


if __name__ == '__main__':

    combine_low_abd_otu_parser = argparse.ArgumentParser()
    combine_low_abd_otu_parser.add_argument('-i', required=True,                            help='input otu count table')
    combine_low_abd_otu_parser.add_argument('-c', required=False, default=0.5,type=float,   help='relative abundance cutoff, default is 0.5')
    combine_low_abd_otu_parser.add_argument('-s', required=False, default=None,             help='interested sample txt')
    combine_low_abd_otu_parser.add_argument('-o', required=True,                            help='output otu table')
    args = vars(combine_low_abd_otu_parser.parse_args())
    combine_low_abd_otu(args)


'''

python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/combine_low_abd_otu.py -i /Users/songweizhi/Desktop/SMP/02_Usearch_BLCA_GTDB_20250325/s07_AllSamples_unoise_otu_table_noEU_mim20000.txt -o /Users/songweizhi/Desktop/SMP/02_Usearch_BLCA_GTDB_20250325/s07_AllSamples_unoise_otu_table_noEU_mim20000_combined_low_abundant_OTUs_0.5.txt -c 0.1

'''



import os
import sys
from Bio import SeqIO

usage = """
FastaSplitler_by_size.py - splits a large FASTA file into smaller subsets by defined size(MB).

Usage:
FastaSplitler_by_size.py [Input_file] [Size_in_MB]
"""

if len(sys.argv) != 3:
    print(usage)
    exit(1)

to_split = sys.argv[1]
size_cutoff_MB_str = sys.argv[2]

# Make sure that the file provided is readable.
try:
    open(to_split, 'r')
except IOError:
    print("Error! " + to_split + " does not exist, or is not readable!")
    exit(1)

# make sure the second argument is a positive value
try:
    float(size_cutoff_MB_str)
except ValueError:
    print("Error! Please provide a positive value!")
    exit(1)
if float(size_cutoff_MB_str) <= 0:
    print("Error! Please provide a positive value!")
    exit(1)


def split_with_size(input_list, size_cutoff):
    new_list = []
    temp_list = []
    sum = 0
    for each in input_list:
        if len(each.seq) >= size_cutoff:
            if temp_list == []:
                # append current element
                temp_list.append(each)
                new_list.append(temp_list)
                # reset temp_list
                temp_list = []
            else:
                # if temp_list != [], append temp_list first
                new_list.append(temp_list)
                # reset temp_list
                temp_list = []
                # then, append new element
                temp_list.append(each)
                new_list.append(temp_list)
                # reset temp_list and sum
                temp_list = []
                sum = 0
        else:
            sum += len(each.seq)
            if sum <= size_cutoff:
                temp_list.append(each)
                pass
            else:
                new_list.append(temp_list)
                temp_list = []
                temp_list.append(each)
                sum = len(each.seq)
    new_list.append(temp_list)
    return new_list


size_cutoff_MB = float(size_cutoff_MB_str)
size_cutoff_bp = 1024 ** 2 * size_cutoff_MB

path, file_name = os.path.split(to_split)
file_name_no_extension = file_name.split('.')[0]
if path == '':
    path = '.'

all_fasta_list = list(SeqIO.parse(to_split, 'fasta'))
new_list = split_with_size(all_fasta_list, size_cutoff_bp)

n = 1
for each_subset in new_list:
    subset = open('%s/%s_%s.fasta' % (path, file_name_no_extension, n), 'w')
    for each_contig in each_subset:
        SeqIO.write(each_contig, subset, 'fasta')
    subset.close()
    n += 1




import os
import glob
import argparse


gapseq_usage = '''
============ gapseq example commands ============

BioSAK gapseq -i gapseq_op_dir -o df.txt
BioSAK gapseq -i gapseq_op_dir -o df.txt -name

=================================================
'''


def sep_path_basename_ext(file_in):

    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(file_name)
    return f_path, f_base, f_ext


def gapseq(args):

    dir_in       = args['i']
    df_out       = args['o']
    include_desc = args['name']

    stdout_file_re = '%s/*_stdout.txt'    % dir_in
    pwy_file_re    = '%s/*-Pathways.tbl'  % dir_in

    stdout_file_list = glob.glob(stdout_file_re)
    pwy_file_list    = glob.glob(pwy_file_re)

    gnm_id_set_stdout = set()
    for each_stdout in stdout_file_list:
        _, f_base, _ = sep_path_basename_ext(each_stdout)
        gnm_id = f_base.split('_stdout')[0]
        gnm_id_set_stdout.add(gnm_id)

    gnm_id_set_pwy = set()
    for each_file in sorted(pwy_file_list):
        _, f_base, _ = sep_path_basename_ext(each_file)
        gnm_id = '-'.join(f_base.split('-')[:-2])
        gnm_id_set_pwy.add(gnm_id)

    # check failed jobs
    failed_gnm_set = set()
    if len (gnm_id_set_stdout) > 0:
        for each_gnm in gnm_id_set_stdout:
            if each_gnm not in gnm_id_set_pwy:
                failed_gnm_set.add(each_gnm)

    # read in annotation results
    gnm_id_set = set()
    detected_pwy_set = set()
    pwy_to_gnm_dict = dict()
    pwy_id_to_name_dict = dict()
    annotation_results_dict = dict()
    for each_file in sorted(pwy_file_list):
        _, f_base, _ = sep_path_basename_ext(each_file)
        gnm_id = '-'.join(f_base.split('-')[:-2])
        gnm_id_set.add(gnm_id)
        encoded_pwy_set = set()
        for each_line in open(each_file):
            if not each_line.startswith('#'):
                if not each_line.startswith('ID\tName'):
                    each_line_split = each_line.strip().split('\t')
                    pwy_id = each_line_split[0]
                    if (pwy_id[0] == '|') and (pwy_id[-1] == '|'):
                        pwy_id = pwy_id[1:-1]
                    pwy_name = each_line_split[1]
                    pwy_id_to_name_dict[pwy_id] = pwy_name
                    pa = each_line_split[2]
                    if pa == 'true':
                        detected_pwy_set.add(pwy_id)
                        encoded_pwy_set.add(pwy_id)
                        if pwy_id not in pwy_to_gnm_dict:
                            pwy_to_gnm_dict[pwy_id] = set()
                        pwy_to_gnm_dict[pwy_id].add(gnm_id)
        annotation_results_dict[gnm_id] = encoded_pwy_set

    pwy_id_list_sorted = sorted(list(detected_pwy_set))
    pwy_id_list_sorted_desc = [('%s__%s' % (i, pwy_id_to_name_dict[i])) for i in pwy_id_list_sorted]

    df_out_handle = open(df_out, 'w')
    if include_desc is False:
        df_out_handle.write('\t' + '\t'.join(pwy_id_list_sorted) + '\n')
    else:
        df_out_handle.write('\t' + '\t'.join(pwy_id_list_sorted_desc) + '\n')
    for each_gnm in sorted(list(gnm_id_set)):
        encoded_pwys = annotation_results_dict[each_gnm]
        value_list = [each_gnm]
        for each_pwy in pwy_id_list_sorted:
            if each_pwy in encoded_pwys:
                value_list.append('1')
            else:
                value_list.append('0')
        df_out_handle.write('\t'.join(value_list) + '\n')
    df_out_handle.close()

    # report genomes failed with GapSeq
    if len(failed_gnm_set) == 1:
        print('It seems that %s was failed with GapSeq, thus was not included in: %s.' % (list(failed_gnm_set)[0], df_out))
    elif len(failed_gnm_set) > 1:
        print('It seems that the following genomes were failed with GapSeq, thus were not included in: %s.' % df_out)
        print('\n'.join(sorted(list(failed_gnm_set))) + '\n')

    print('Done!')


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument('-i',       required=True,                        help='folder holds the *Pathways.tbl files')
    parser.add_argument('-o',       required=True,                        help='output file')
    parser.add_argument('-name',    required=False, action="store_true",  help='include pathway name in the output dataframe')
    args = vars(parser.parse_args())
    gapseq(args)



import os
import math
import argparse
from Bio import SeqIO

split_fasta_usage = '''
============= split_fasta example command =============

BioSAK split_fasta -i input.fa -o output_dir
BioSAK split_fasta -i input.fa -o output_dir -ns 200
BioSAK split_fasta -i input.fa -o output_dir -nf 10

=======================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_ext = os.path.splitext(file_name)

    return file_path, file_basename, file_ext


def split_fasta(args):

    fasta_in         = args['i']
    per_file_seq_num = args['ns']
    num_of_file      = args['nf']
    output_dir       = args['o']

    fasta_in_path, fasta_in_basename, fasta_in_ext = sep_path_basename_ext(fasta_in)

    if (per_file_seq_num is None) and (num_of_file is None):
        print('Please either use the "-ns" flag to define the number of sequence per file or the "-nf" flag to specify the number of files to be generated.')
        exit()
    elif (per_file_seq_num is not None) and (num_of_file is not None):
        print('Options "-ns" and "-nf" are incompatible, only specify one of them.')
        exit()
    elif (per_file_seq_num is not None) and (num_of_file is None):

        if os.path.isdir(output_dir) is True:
            print('Output folder detected, please provide a different name!')
            exit()
        else:
            os.mkdir(output_dir)

        n = 1
        for seq_record in SeqIO.parse(fasta_in, 'fasta'):
            file_index = (n - 1) // per_file_seq_num + 1
            pwd_sub_file = '%s/%s_%s.fa' % (output_dir, fasta_in_basename, file_index)
            if per_file_seq_num == 1:
                pwd_sub_file = '%s/%s.fa' % (output_dir, seq_record.id)
            # write out sequence
            with open(pwd_sub_file, 'a') as pwd_sub_file_handle:
                pwd_sub_file_handle.write('>%s\n' % seq_record.id)
                pwd_sub_file_handle.write('%s\n'  % str(seq_record.seq))
            n += 1
    elif (per_file_seq_num is None) and (num_of_file is not None):

        if os.path.isdir(output_dir) is True:
            print('Output folder detected, please provide a different name!')
            exit()
        else:
            os.mkdir(output_dir)

        # get total number of sequences
        total_seq_num = 0
        for each_seq in SeqIO.parse(fasta_in, 'fasta'):
            total_seq_num += 1
        seq_num_per_file = math.ceil(total_seq_num/num_of_file)

        # write out
        seq_index = 1
        for seq_record in SeqIO.parse(fasta_in, 'fasta'):
            file_index = math.ceil(seq_index/seq_num_per_file)
            pwd_sub_file = '%s/%s_%s.fa' % (output_dir, fasta_in_basename, file_index)

            # write out sequence
            with open(pwd_sub_file, 'a') as pwd_sub_file_handle:
                pwd_sub_file_handle.write('>%s\n' % seq_record.id)
                pwd_sub_file_handle.write('%s\n'  % str(seq_record.seq))
            seq_index += 1


if __name__ == '__main__':

    split_fasta_parser = argparse.ArgumentParser(usage=split_fasta_usage)
    split_fasta_parser.add_argument('-i',  required=True,                          help='input fasta file')
    split_fasta_parser.add_argument('-o',  required=True,                          help='output dir')
    split_fasta_parser.add_argument('-ns', required=False, default=None, type=int, help='number of sequences per file')
    split_fasta_parser.add_argument('-nf', required=False, default=None, type=int, help='number of files to be generated')
    args = vars(split_fasta_parser.parse_args())
    split_fasta(args)



import os
import argparse


SankeyTaxon_parser_usage ='''
===================================== SankeyTaxon example commands ====================================

module load R

# Visualize taxonomic classification at phylum, class and genus level
BioSAK SankeyTaxon -taxon KelpBins_gtdbtk.tsv -r pcg -p KelpBins

# Visualize taxonomic classification at phylum and genus level, with unclear classifications excluded 
BioSAK SankeyTaxon -taxon KelpBins_gtdbtk.tsv -r pg -p KelpBins -ec

=======================================================================================================
'''

'''
module load python/3.5.2
module load R

BioSAK SankeyTaxon -taxon KelpBins_gtdbtk.tsv -r pcof -p KelpBins
BioSAK SankeyTaxon -taxon KelpBins_gtdbtk.tsv -r pcof -p KelpBins -ec

python3 ~/PycharmProjects/BioSAK/BioSAK/SankeyTaxonClassification.py -taxon KelpBins_gtdbtk.tsv -r pcof -p KelpBins
python3 ~/PycharmProjects/BioSAK/BioSAK/SankeyTaxonClassification.py -taxon KelpBins_gtdbtk.tsv -r pcof -p KelpBins -ec
'''


def unique_list_elements(list_input):

    list_output = []
    for each_element in list_input:
        if each_element not in list_output:
            list_output.append(each_element)

    return list_output


def SankeyTaxon(args):

    GTDB_output =                       args['taxon']
    taxon_ranks =                       args['r']
    output_prefix =                     args['p']
    plot_width =                        args['x']
    plot_height =                       args['y']
    explicit_classifications_only =     args['ec']


    pwd_self = os.path.realpath(__file__)
    self_path = '/'.join(pwd_self.split('/')[:-1])
    pwd_get_sankey_plot_R = '%s/sankey.R' % self_path

    output_file_txt = '%s_sankey_taxon.txt' % output_prefix

    rank_full_list = ['d', 'p', 'c', 'o', 'f', 'g', 's']
    rank_pos_dict = {'d': 0, 'p': 1, 'c': 2, 'o': 3, 'f': 4, 'g': 5, 's': 6}

    ranks_to_plot = []
    for each_rank in rank_full_list:
        if each_rank in taxon_ranks:
            if each_rank not in ranks_to_plot:
                ranks_to_plot.append(each_rank)


    paired_taxon_list_all = []
    genome_number = 0
    for each_genome_taxon in open(GTDB_output):
        if not each_genome_taxon.startswith('user_genome'):

            genome_number += 1

            genome_taxon_split = each_genome_taxon.strip().split('\t')

            taxon_asign_full = []
            if len(genome_taxon_split) == 1:
                taxon_asign_full = ['d__', 'p__', 'c__', 'o__', 'f__', 'g__', 's__']

            elif (len(genome_taxon_split) > 1) and (';' in genome_taxon_split[1]):
                assignment = genome_taxon_split[1].split(';')
                if len(assignment) == 7:
                    taxon_asign_full = assignment
                if len(assignment) == 6:
                    taxon_asign_full = assignment + ['s__']
                if len(assignment) == 5:
                    taxon_asign_full = assignment + ['g__', 's__']
                if len(assignment) == 4:
                    taxon_asign_full = assignment + ['f__', 'g__', 's__']
                if len(assignment) == 3:
                    taxon_asign_full = assignment + ['o__', 'f__', 'g__', 's__']
                if len(assignment) == 2:
                    taxon_asign_full = assignment + ['c__', 'o__', 'f__', 'g__', 's__']

            elif (len(genome_taxon_split) > 1) and (';' not in genome_taxon_split[1]):
                taxon_asign_full = [genome_taxon_split[1]] + ['p__', 'c__', 'o__', 'f__', 'g__', 's__']


            rank_to_plot_paired = []
            n = 0
            while n < len(ranks_to_plot) -1:
                rank_to_plot_paired.append([ranks_to_plot[n], ranks_to_plot[n + 1]])
                n += 1


            for each_pair in rank_to_plot_paired:

                each_pair_taxon_left = taxon_asign_full[rank_pos_dict[each_pair[0]]]
                each_pair_taxon_right = taxon_asign_full[rank_pos_dict[each_pair[1]]]
                each_pair_taxon = '%s,%s' % (each_pair_taxon_left, each_pair_taxon_right)

                # combine unclear classifications
                if explicit_classifications_only is False:
                    paired_taxon_list_all.append(each_pair_taxon)

                # don't show unclear classification
                else:
                    if (each_pair_taxon_left.endswith('__') is False) and (each_pair_taxon_right.endswith('__') is False):
                        paired_taxon_list_all.append(each_pair_taxon)


    paired_taxon_list_uniq = unique_list_elements(paired_taxon_list_all)
    paired_taxon_list_uniq_count_dict = {}
    for each_key in paired_taxon_list_uniq:
        paired_taxon_list_uniq_count_dict[each_key] = paired_taxon_list_all.count(each_key)


    output_file_handle = open(output_file_txt, 'w')
    output_file_handle.write('C1,C2,Number\n')
    for each_genome_taxon in paired_taxon_list_uniq_count_dict:
        for_out = '%s,%s\n' % (each_genome_taxon, paired_taxon_list_uniq_count_dict[each_genome_taxon])
        output_file_handle.write(for_out)
    output_file_handle.close()


    # run Rscript

    if plot_width == None:
        if len(ranks_to_plot) <= 5:
            plot_width = (len(ranks_to_plot) - 1) * 300
        else:
            plot_width = (len(ranks_to_plot)-1)*250

    if plot_height == None:
        plot_height = genome_number * 2
        if plot_height < 600:
            plot_height = 600

    print('Rscript %s -f %s -x %s -y %s' % (pwd_get_sankey_plot_R, output_file_txt, plot_width, plot_height))
    os.system('Rscript %s -f %s -x %s -y %s' % (pwd_get_sankey_plot_R, output_file_txt, plot_width, plot_height))


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='', add_help=True)

    parser.add_argument('-taxon', required=True,  help='taxon classification results')
    parser.add_argument('-r',     required=True,  help='taxon ranks to plot, e.g. dpcofgs, pco, pcf, cfs')
    parser.add_argument('-p',     required=True,  help='output prefix')
    parser.add_argument('-ec',    required=False, action='store_true', help='only plot explicit classifications')
    parser.add_argument('-x',     required=False, type=int, help='plot width')
    parser.add_argument('-y',     required=False, type=int, help='plot height')

    args = vars(parser.parse_args())

    SankeyTaxon(args)




import os
import glob
import argparse
from Bio import SeqIO


abund_usage = '''
========================== mask example commands ==========================

BioSAK mask -h
BioSAK abund -h

cd /Users/songweizhi/Desktop
python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/abund.py -i symbiont_genomes -x fna -o symbiont_genomes_abund -f 

python3 abund.py -i symbiont_genomes -x fna -o symbiont_genomes_abund -f -t 6

===========================================================================
'''

def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def mask(args):

    gnm_dir         = args['i']
    file_ext        = args['x']

def mapping(fq_r1, fq_r2, ref_seq, op_prefix, num_threads):

    bwa_cmd             = 'bwa mem -5SP -t %s %s %s %s | samblaster > %s.sam'                                                           % (num_threads, ref_seq, fq_r1, fq_r2, op_prefix)
    samtools_view_cmd   = 'samtools view -@ 32 -bS -h -b %s.sam > %s.bam'                                                               % (op_prefix, op_prefix)
    samtools_sort_cmd   = 'samtools sort -@ 32 %s.bam -o %s.sorted.bam'                                                                 % (op_prefix, op_prefix)
    bamm_filter_cmd     = 'bamm filter -b %s.sorted.bam --percentage_id 0.99 --percentage_aln 0.9'                                      % (op_prefix)
    coverm_filter_cmd   = 'coverm filter -b %s.sorted.bam --min-read-aligned-percent 0.9 --min-read-percent-identity 0.99'              % (op_prefix)
    pileup_sh_cmd       = 'pileup.sh in=%s.sorted_filtered.bam out=%s.sorted_filtered.cov rpkm=%s.sorted_filtered.rpkm overwrite=true'  % (op_prefix, op_prefix, op_prefix)
    seqkit_stat_cmd     = 'seqkit stat %s.fastq > %s.stat'                                                                              % (op_prefix, op_prefix)

    os.system(bwa_cmd)
    os.system(samtools_view_cmd)
    os.system(samtools_sort_cmd)
    os.system(coverm_filter_cmd)
    os.system(pileup_sh_cmd)
    os.system(seqkit_stat_cmd)


def get_rpkm():
    pass


def abund(args):

    gnm_dir             = args['i']
    file_ext            = args['x']
    output_folder       = args['o']
    num_threads         = args['t']
    force_create_op_dir = args['f']

    gnm_file_re = '%s/*.%s' % (gnm_dir, file_ext)
    input_gnm_list = glob.glob(gnm_file_re)

    # create output folder
    if os.path.isdir(output_folder) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % output_folder)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % output_folder)

    concatenated_renamed_gnm = '%s/combined.fasta' % output_folder

    concatenated_renamed_fasta_handle = open(concatenated_renamed_gnm, 'w')
    gnm_name_dict_old_to_new = dict()
    gnm_name_dict_new_to_old = dict()
    for each_gnm in input_gnm_list:
        gnm_name, gnm_path, gnm_base, gnm_ext = sep_path_basename_ext(each_gnm)
        gnm_name_new = gnm_base
        gnm_name_new = gnm_name_new.replace('_', '').replace('-', '')
        gnm_name_dict_old_to_new[gnm_base] = gnm_name_new
        gnm_name_dict_new_to_old[gnm_name_new] = gnm_base
        seq_index = 1
        for each_seq in SeqIO.parse(each_gnm, 'fasta'):
            concatenated_renamed_fasta_handle.write('>%s_%s\n' % (gnm_name_new, seq_index))
            concatenated_renamed_fasta_handle.write('%s\n' % each_seq.seq)
            seq_index += 1
    concatenated_renamed_fasta_handle.close()

    metaxa2_ssu_cmd = 'metaxa2 --plus --mode m --cpu 12 --multi_thread T --table T -g ssu --not_found T -i %s -o %s.metaxa2_ssu' % (concatenated_renamed_gnm, concatenated_renamed_gnm)
    metaxa2_lsu_cmd = 'metaxa2 --plus --mode m --cpu 12 --multi_thread T --table T -g lsu --not_found T -i %s -o %s.metaxa2_lsu' % (concatenated_renamed_gnm, concatenated_renamed_gnm)
    os.system(metaxa2_ssu_cmd)
    os.system(metaxa2_lsu_cmd)




if __name__ == '__main__':

    abund_parser = argparse.ArgumentParser(usage=abund_usage)
    abund_parser.add_argument('-i', required=True,                          help='input genome directory')
    abund_parser.add_argument('-x', required=True,                          help='file extension')
    abund_parser.add_argument('-o', required=True,                          help='output directory')
    abund_parser.add_argument('-f', required=False, action="store_true",    help='force overwrite')
    abund_parser.add_argument('-t', required=False, type=int, default=1,    help='number of threads')
    args = vars(abund_parser.parse_args())
    abund(args)


'''
cd /scratch/PI/ocessongwz/Sponge_r220/7_symbiont_abundance/symbiont_genomes
metaxa2 --plus --mode m --cpu 12 --multi_thread T --table T -g ssu --not_found T -i VYC.fna -o VYC.metaxa2_ssu
metaxa2 --plus --mode m --cpu 12 --multi_thread T --table T -g lsu --not_found T -i VYC.fna -o VYC.metaxa2_lsu
cut -f 1,9,10 VYC.metaxa2_ssu.extraction.results > masked_metaxa_ssu.bed
cut -f 1,9,10 VYC.metaxa2_lsu.extraction.results > masked_metaxa_lsu.bed
cat masked_metaxa_ssu.bed masked_metaxa_lsu.bed > masked_metaxa.bed
barrnap --kingdom bac --threads 32 --reject 0.3 VYC.fna > out_bac.gff
barrnap --kingdom arc --threads 32 --reject 0.3 VYC.fna > out_arc.gff
barrnap --kingdom euk --threads 32 --reject 0.3 VYC.fna > out_euk.gff
cut -f 1,4,5 out_bac.gff >> masked_barrnap.bed
cut -f 1,4,5 out_arc.gff >> masked_barrnap.bed
cut -f 1,4,5 out_euk.gff >> masked_barrnap.bed
cat masked_barrnap.bed masked_metaxa.bed > masked_rrna.bed

# run tRNAscan-SE
tRNAscan-SE -A -b VYC.fna.ar.bedformat --thread 1 VYC.fna
tRNAscan-SE -B -b VYC.fna.bac.bedformat --thread 1 VYC.fna
cat VYC.fna.ar.bedformat VYC.fna.bac.bedformat > VYC.fna.bedformat
cut -f 1,2,3 VYC.fna.bedformat > masked_trnascan.bed

# Find duplicate areas
dustmasker -in VYC.fna -out VYC.fna.lowcom.out -outfmt acclist
cut -d '>' -f 2 VYC.fna.lowcom.out > masked_dustmasker.bed 

# Cover the above area with NNNN
cat masked_rrna.bed masked_trnascan.bed masked_dustmasker.bed > VYC.fna_mask.bed
awk -F'\t' '$2>=0' VYC.fna_mask.bed > VYC.fna_mask_0.bed
bedtools maskfasta -fi VYC.fna -bed VYC.fna_mask_0.bed -fo VYC.fna_masked.fasta
'''



import os
import glob
import argparse
from Bio import SeqIO
from Bio.Seq import Seq
import multiprocessing as mp


gbk2ffn_usage = '''
========= gbk2ffn example commands =========

BioSAK gbk2ffn -i MAG.gbk -o MAG.ffn
BioSAK gbk2ffn -i gbk_dir -o ffn_dir -t 6

============================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    # separate file basename and extension
    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


def gbk2ffn_single(arg_list):

    gbk_in  = arg_list[0]
    ffn_out = arg_list[1]

    ffn_out_handle = open(ffn_out, 'w')
    for seq_record in SeqIO.parse(gbk_in, 'genbank'):
        sequence_str = str(seq_record.seq)
        for feature in seq_record.features:
            if feature.type == 'CDS':
                feature_loci = feature.location
                feature_loci_start = feature_loci.start
                feature_loci_end = feature_loci.end
                feature_loci_strand = feature_loci.strand
                coding_seq = sequence_str[feature_loci_start:feature_loci_end]
                feature_locus_tag = feature.qualifiers['locus_tag'][0]
                coding_seq_5_3 = coding_seq
                if feature_loci_strand == -1:
                    coding_seq_5_3 = str(Seq(coding_seq).reverse_complement())
                ffn_out_handle.write('>%s\n' % feature_locus_tag)
                ffn_out_handle.write('%s\n' % coding_seq_5_3)
    ffn_out_handle.close()


def gbk2ffn(args):

    gbk_in      = args['i']
    in_ext      = args['ix']
    ffn_out     = args['o']
    out_ext     = args['ox']
    num_threads = args['t']

    if os.path.isfile(gbk_in) is True:
        gbk2ffn_single([gbk_in, ffn_out])

    elif os.path.isdir(gbk_in) is True:

        gbk_file_re   = '%s/*.%s' % (gbk_in, in_ext)
        gbk_file_list = glob.glob(gbk_file_re)

        if len(gbk_file_list) == 0:
            print('No input gbk found, program exited!')
            exit()

        if os.path.isdir(ffn_out) is True:
            print('output folder detected, program exited!')
            exit()
        os.system('mkdir %s' % ffn_out)

        mp_arg_lol = []
        for each_gbk in gbk_file_list:
            gbk_path, gbk_base, gbk_ext = sep_path_basename_ext(each_gbk)
            pwd_ffn_out = '%s/%s.%s' % (ffn_out, gbk_base, out_ext)
            mp_arg_lol.append([each_gbk, pwd_ffn_out])

        pool = mp.Pool(processes=num_threads)
        pool.map(gbk2ffn_single, mp_arg_lol)
        pool.close()
        pool.join()
    else:
        print('No input gbk found, program exited!')
        exit()


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument('-i',  required=True,                       help='input gbk')
    parser.add_argument('-ix', required=False, default='gbk',       help='input file extension, default: gbk')
    parser.add_argument('-o',  required=True,                       help='output ffn')
    parser.add_argument('-ox', required=False, default='ffn',       help='output file extension, default: ffn')
    parser.add_argument('-t',  required=False, type=int, default=1, help='number of threads')
    args = vars(parser.parse_args())
    gbk2ffn(args)



import argparse
import pandas as pd


merge_df_usage = '''
================ merge_df example commands ================

BioSAK merge_df -1 df_1.txt -2 df_2.txt -o merged_df.txt

/Library/Frameworks/Python.framework/Versions/3.10/bin/python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/merge_df.py -1 df_1.txt -2 df_2.txt -o merged_df.txt

===========================================================
'''


def merge_df(args):

    df1_txt             = args['1']
    df2_txt             = args['2']
    file_out            = args['o']
    df_separator        = args['s']
    column_name_pos     = 0
    row_name_pos        = 0

    # setup separator
    if df_separator in ['tab', 'Tab', 'TAB']:
        sep_symbol = '\t'
    elif df_separator in ['comma', 'Comma', 'COMMA']:
        sep_symbol = ','
    else:
        print('Please specify separator as either tab or comma, program exited!')
        exit()

    df1 = pd.read_csv(df1_txt, sep=sep_symbol, header=column_name_pos, index_col=row_name_pos)
    df2 = pd.read_csv(df2_txt, sep=sep_symbol, header=column_name_pos, index_col=row_name_pos)

    frames = [df1, df2]
    merged_df = pd.concat(frames, axis=1)

    merged_df.to_csv(file_out, sep=sep_symbol, na_rep='na')


if __name__ == '__main__':

    merge_df_parser = argparse.ArgumentParser(usage=merge_df_usage)
    merge_df_parser.add_argument('-1',     required=True,                          help='the first input dataframe')
    merge_df_parser.add_argument('-2',     required=True,                          help='the second input dataframe')
    merge_df_parser.add_argument('-o',     required=True,                          help='output file')
    merge_df_parser.add_argument('-s',     required=False, default='tab',          help='column separator, choose from tab and comma, default: tab')
    args = vars(merge_df_parser.parse_args())
    merge_df(args)



from typing import List

from ncbi.datasets.openapi import ApiClient as DatasetsApiClient
from ncbi.datasets.openapi import ApiException as DatasetsApiException
from ncbi.datasets.openapi import GenomeApi as DatasetsGenomeApi

# 	SB0662_bin_1	Chloroflexi bacterium
accessions = ['GCA_009840555.1', 'GCA_009840575.1', 'GCA_009840525.1']
zipfile = '/Users/songweizhi/Desktop/SB0662_bin_1.zip'




def genome_download_by_asm_accessions(genome_assembly_accessions: List[str],
                                      zipfile_name: str):
    """Downloads an NCBI Datasets Genome Data Package

    Args:
        genome_assembly_accessions: A list of NCBI assembly accessions
        zipfile_name: output file name
    """
    with DatasetsApiClient() as api_client:
        genome_api = DatasetsGenomeApi(api_client)
        try:
            print('Begin download of genome data package ...')
            genome_ds_download = genome_api.download_assembly_package(
                genome_assembly_accessions,
                include_annotation_type=['RNA_FASTA', 'PROT_FASTA'],
                _preload_content=False)

            with open(zipfile_name, 'wb') as f:
                f.write(genome_ds_download.data)
            print(f'Download completed -- see {zipfile_name}')
        except DatasetsApiException as e:
            print(f'Exception when calling download_assembly_package: {e}\n')

#
# genome_download_by_asm_accessions(accessions, zipfile)



needed_id = set()
for each in open('/Users/songweizhi/Desktop/64genomes.txt'):
    needed_id.add(each.strip())
print(needed_id)

for each in open('/Users/songweizhi/Desktop/prokaryotes-2.csv'):

    assembly_id = each.strip().split(',')[5][1:-1]
    if assembly_id in needed_id:
        print(each.strip())



import os
import argparse


sam2bam_usage = '''
============== sam2bam example command ==============

module load samtools
BioSAK sam2bam -sam input.sam -t 12

# This is a wrapper for the following three steps:
samtools view -bS input.sam -o input.bam
samtools sort input.bam -o input_sorted.bam
samtools index input_sorted.bam

=====================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_ext = os.path.splitext(file_name)

    return file_path, file_basename, file_ext


def sam2bam(args):

    sam_in     = args['sam']
    thread_num = args['t']

    sam_path, sam_basename, sam_ext = sep_path_basename_ext(sam_in)

    pwd_bam             = '%s/%s.bam'                               % (sam_path, sam_basename)
    pwd_bam_sorted      = '%s/%s_sorted.bam'                        % (sam_path, sam_basename)
    cmd_samtools_view   = 'samtools view -bS %s -o %s --threads %s' % (sam_in, pwd_bam, thread_num)
    cmd_samtools_sort   = 'samtools sort %s -o %s --threads %s'     % (pwd_bam, pwd_bam_sorted, thread_num)
    cmd_samtools_index  = 'samtools index %s'                       % (pwd_bam_sorted)

    print('Converting sam to bam')
    os.system(cmd_samtools_view)
    print('Sorting bam file')
    os.system(cmd_samtools_sort)
    print('Indexing sorted bam file')
    os.system(cmd_samtools_index)
    print('Removing unsorted bam file')
    os.system('rm %s' % pwd_bam)
    print('All done!')

if __name__ == '__main__':

    sam2bam_parser = argparse.ArgumentParser(usage=sam2bam_usage)
    sam2bam_parser.add_argument('-sam', required=True,                       help='sam file')
    sam2bam_parser.add_argument('-t',   required=False, type=int, default=1, help='number of threads')
    args = vars(sam2bam_parser.parse_args())
    sam2bam(args)



import os
import glob
import math
import argparse
import pandas as pd
from functools import reduce


get_abd3_stats_usage = '''
===================== get_abd3_stats example commands =====================

BioSAK get_abd3_stats -i get_abd2_mapping_op_dir -o op_dir -f 
BioSAK get_abd3_stats -i cov_rpkm_stat_files -o get_abd3_stats_op_dir -f 

# Note
0 becomes na after taking log.

===========================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def get_abd3_stats(args):

    rpkm_stat_cov_dir   = args['i']
    op_dir              = args['o']
    force_overwrite     = args['f']

    # check input files
    if os.path.isdir(rpkm_stat_cov_dir) is False:
        print('%s not found, program exited!' % rpkm_stat_cov_dir)
        exit()

    rpkm_file_re    = '%s/*.rpkm' % rpkm_stat_cov_dir
    stat_file_re    = '%s/*.stat' % rpkm_stat_cov_dir
    rpkm_file_list  = glob.glob(rpkm_file_re)
    stat_file_list  = glob.glob(stat_file_re)

    if len(rpkm_file_list) == 0:
        print('.rpkm file not found, program exited!')
        exit()

    if len(stat_file_list) == 0:
        print('.stat file not found, program exited!')
        exit()

    # define output file name
    gnm_level_rpkm_dir       = '%s/genome_level_rpkm'   % op_dir
    combined_rpkm_file       = '%s/combined.rpkm'       % op_dir
    combined_rpkm_file_log10 = '%s/combined.log10.rpkm' % op_dir

    # create op dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output directory detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s' % gnm_level_rpkm_dir)

    # read in stat file
    sample_total_read_num_dict = dict()
    for stat_file in sorted(stat_file_list):
        _, _, stat_base, _ = sep_path_basename_ext(stat_file)
        stat_file_lines = open(stat_file).readlines()
        info_line_split = stat_file_lines[-1].strip().split(' ')
        while '' in info_line_split:
            info_line_split.remove('')
        read_num = int(info_line_split[3].replace(',', ''))
        sample_total_read_num_dict[stat_base] = read_num

    # summarise rpkm on genome level
    sample_id_list = []
    for rpkm_file in sorted(rpkm_file_list):
        rpkm_name, rpkm_path, rpkm_base, rpkm_ext = sep_path_basename_ext(rpkm_file)
        sample_id = rpkm_base.split('.sorted_filtered')[0]
        sample_id_list.append(sample_id)
        op_txt = '%s/%s.cal_bin.rpkm' % (gnm_level_rpkm_dir, sample_id)
        rpkm_df = pd.read_csv(rpkm_file, sep='\t', header=4)
        subset_df_col_list = ['#Name', 'Length', 'Reads']
        sample_total_read_num = sample_total_read_num_dict[sample_id]
        subset_df = rpkm_df.loc[:, subset_df_col_list]
        subset_df['bin_name'] = subset_df['#Name'].str.rsplit('_', n=1, expand=True)[0]
        subset_df['Length'] = subset_df['Length'].astype(int)
        subset_df['Reads'] = subset_df['Reads'].astype(int)

        # calulate RPKM
        op_df = subset_df.groupby(["bin_name"])[['Length', 'Reads']].sum()
        op_df[sample_id] = (op_df['Reads']*1000000000)/(op_df['Length']*sample_total_read_num*2)
        op_df.to_csv(op_txt, sep='\t')

    # combine the results
    rpkm_df_list = []
    for sample_id in sample_id_list:
        gnm_level_rpkm_file = '%s/%s.cal_bin.rpkm' % (gnm_level_rpkm_dir, sample_id)
        gnm_level_rpkm_df = pd.read_csv(gnm_level_rpkm_file, sep='\t')
        gnm_level_rpkm_df = gnm_level_rpkm_df.drop(["Length", "Reads"], axis=1)
        rpkm_df_list.append(gnm_level_rpkm_df)

    # write out combined_rpkm_file
    rpkm_df_combined = reduce(lambda left, right: pd.merge(left, right, on='bin_name',how='outer'), rpkm_df_list)
    rpkm_df_combined.to_csv(combined_rpkm_file, sep='\t',index=False)

    # write out combined_rpkm_file_log10
    combined_rpkm_file_log10_handle = open(combined_rpkm_file_log10, 'w')
    for each_line in open(combined_rpkm_file):
        each_line_split = each_line.strip().split('\t')
        if each_line.startswith('bin_name'):
            combined_rpkm_file_log10_handle.write(each_line)
        else:
            value_list = [each_line_split[0]]
            for each_value in each_line_split[1:]:
                each_value = float(each_value)
                if each_value == 0:
                    value_list.append('na')
                else:
                    each_value_log = math.log(each_value)
                    value_list.append(each_value_log)
            combined_rpkm_file_log10_handle.write('%s\n' % '\t'.join([str(i) for i in value_list]))
    combined_rpkm_file_log10_handle.close()

    # final report
    print('Done!')


if __name__ == '__main__':

    get_abd3_stats_parser = argparse.ArgumentParser()
    get_abd3_stats_parser.add_argument('-i', required=True,                          help='output directory from get_abd2_mapping')
    get_abd3_stats_parser.add_argument('-o', required=True,                          help='output directory')
    get_abd3_stats_parser.add_argument('-f', required=False, action="store_true",    help='force overwrite')
    args = vars(get_abd3_stats_parser.parse_args())
    get_abd3_stats(args)



import argparse


ezaai2mat_usage = '''
============== ezaai2mat example command ==============

BioSAK ezaai2mat -i ezaai.table -o ezaai_matrix.txt

=======================================================
'''


def ezaai2mat(args):

    ezaai_table = args['i']
    op_matrix   = args['o']

    seq_set = set()
    aai_dict = dict()
    for each_line in open(ezaai_table):
        if not each_line.startswith('ID 1	ID 2'):
            each_line_split = each_line.strip().split('\t')
            seq_1 = each_line_split[2]
            seq_2 = each_line_split[3]
            aai_value = each_line_split[4]
            key_str = '%s__|__%s' % (seq_1, seq_2)
            aai_dict[key_str] = aai_value
            seq_set.add(seq_1)
            seq_set.add(seq_2)

    seq_list_sorted = sorted([i for i in seq_set])

    op_matrix_handle = open(op_matrix, 'w')
    op_matrix_handle.write('\t' + '\t'.join(seq_list_sorted) + '\n')
    for seq1 in seq_list_sorted:
        aai_list = [seq1]
        for seq2 in seq_list_sorted:
            key_str = '%s__|__%s' % (seq1, seq2)
            aai = aai_dict.get(key_str, 'na')
            aai_list.append(aai)
        op_matrix_handle.write('\t'.join(aai_list) + '\n')
    op_matrix_handle.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-i', required=True, help='ezaai produced table')
    parser.add_argument('-o', required=True, help='output AAI matrix')
    args = vars(parser.parse_args())
    ezaai2mat(args)



import argparse


compare_sets_usage = '''
========= compare_sets example command =========

BioSAK compare_sets -1 file1.txt -2 file2.txt

================================================
'''


def compare_sets(args):

    file_1 = args['1']
    file_2 = args['2']

    set_1 = set()
    for each_line in open(file_1):
        set_1.add(each_line.strip().split()[0])

    set_2 = set()
    for each_line in open(file_2):
        set_2.add(each_line.strip().split()[0])

    shared_id_set = set(set_1).intersection(set_2)

    set_1_uniq = set()
    for e1 in set_1:
        if e1 not in shared_id_set:
            set_1_uniq.add(e1)

    set_2_uniq = set()
    for e2 in set_2:
        if e2 not in shared_id_set:
            set_2_uniq.add(e2)

    if len(shared_id_set) == 0:
        print('No id shared between %s and %s' % (file_1, file_2))
    else:
        print('Shared IDs (%s)\n%s\n' % (len(shared_id_set), '\n'.join(shared_id_set)))
        print('IDs uniq to %s (%s)\n%s\n' % (file_1, len(set_1_uniq), '\n'.join(sorted(list(set_1_uniq)))))
        print('IDs uniq to %s (%s)\n%s\n' % (file_2, len(set_2_uniq), '\n'.join(sorted(list(set_2_uniq)))))


if __name__ == '__main__':

    compare_sets_parser = argparse.ArgumentParser(usage=compare_sets_usage)
    compare_sets_parser.add_argument('-1', required=True, help='file 1')
    compare_sets_parser.add_argument('-2', required=True, help='file 2')
    args = vars(compare_sets_parser.parse_args())
    compare_sets(args)



import os
import argparse


submitHPC_usage = '''
============= submitHPC example commands =============

BioSAK submitHPC -c cmds.txt -t 6 -p ale

# need to have submitHPC.sh in system path
submitHPC.sh --cmd "$cmd" -n 6 -c ale_1

======================================================
'''


def submitHPC(args):

    js_prefix   = args['p']
    cmds_txt    = args['c']
    thread_num  = args['t']

    js_index = 1
    for each_cmd in open(cmds_txt):
        cmd_str = each_cmd.strip()
        submit_cmd = 'submitHPC.sh --cmd "%s" -n %s -c %s_%s' % (cmd_str, thread_num, js_prefix, js_index)
        js_index += 1
        os.system(submit_cmd)


if __name__ == '__main__':

    # arguments for COG_parser
    submitHPC_parser = argparse.ArgumentParser(usage=submitHPC_usage)
    submitHPC_parser.add_argument('-p',               required=True,                              help='js prefix')
    submitHPC_parser.add_argument('-c',               required=True,                              help='commands txt')
    submitHPC_parser.add_argument('-t',               required=False, type=int, default=1,        help='number of threads')
    args = vars(submitHPC_parser.parse_args())
    submitHPC(args)



import os
import glob
import argparse
from Bio import SeqIO
from dna_features_viewer import GraphicFeature, GraphicRecord
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


DnaFeaturesViewer_usage = '''
============================ DnaFeaturesViewer example commands ============================

BioSAK DnaFeaturesViewer -target shc -gbk gbk_dir -x gbk -o demo.pdf -c gene_color.txt -ft

============================================================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    # separate file basename and extension
    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


def gbk_to_feature_list(input_gbk, target_locus_tag_list, flk_len_to_plot, forward_strand_target, gene_color_dict):

    # get contigs to plot region dict
    ctg_to_plot_region_dict = dict()
    ctg_to_plot_region_len_dict = dict()
    locus_tag_to_direction_dict = dict()
    for seq_record in SeqIO.parse(input_gbk, 'genbank'):
        contig_id  = seq_record.id
        contig_len = len(seq_record.seq)
        for seq_feature in seq_record.features:
            if seq_feature.type == "CDS":
                locus_tag = seq_feature.qualifiers['locus_tag'][0]
                if locus_tag in target_locus_tag_list:

                    pos_start = seq_feature.location.start
                    pos_end = seq_feature.location.end
                    feature_strand = seq_feature.strand
                    locus_tag_to_direction_dict[locus_tag] = feature_strand

                    to_plot_start = pos_start - flk_len_to_plot
                    if to_plot_start < 1:
                        to_plot_start = 1

                    to_plot_end = pos_end + flk_len_to_plot
                    if to_plot_end > contig_len:
                        to_plot_end = contig_len

                    if contig_id not in ctg_to_plot_region_dict:
                        ctg_to_plot_region_dict[contig_id] = dict()
                        ctg_to_plot_region_len_dict[contig_id] = dict()

                    ctg_to_plot_region_dict[contig_id][locus_tag] = [to_plot_start, to_plot_end]
                    ctg_to_plot_region_len_dict[contig_id][locus_tag] = to_plot_end - to_plot_start + 1

    to_write_feature_list = []
    for seq_record in SeqIO.parse(input_gbk, 'genbank'):
        gbk_path, gbk_basename, gbk_ext = sep_path_basename_ext(input_gbk)
        contig_id = seq_record.id
        if contig_id in ctg_to_plot_region_dict:
            current_ctg_to_plot_region_dict = ctg_to_plot_region_dict[contig_id]
            current_ctg_to_plot_region_len_dict = ctg_to_plot_region_len_dict[contig_id]
            for target_region in current_ctg_to_plot_region_dict:
                to_plot_region = current_ctg_to_plot_region_dict[target_region]
                to_plot_region_len = current_ctg_to_plot_region_len_dict[target_region]
                target_region_direction = locus_tag_to_direction_dict[target_region]

                reverse_feature = False
                if forward_strand_target is True:
                    if target_region_direction == -1:
                        reverse_feature = True
                else:
                    if target_region_direction == 1:
                        reverse_feature = True

                location_label = '%s___%s___%s' % (gbk_basename, contig_id, target_region)
                for seq_feature in seq_record.features:
                    if seq_feature.type == "CDS":
                        locus_tag      = seq_feature.qualifiers['locus_tag'][0]
                        gene_name      = seq_feature.qualifiers.get('gene', ['na'])[0]
                        pos_start      = seq_feature.location.start
                        pos_end        = seq_feature.location.end
                        feature_strand = seq_feature.strand
                        gene_color     = gene_color_dict.get(gene_name, '')

                        to_label = gene_name
                        if to_label == 'na':
                            to_label = locus_tag

                        strand_index = ''
                        strand_index_reverse = ''
                        if feature_strand == 1:
                            strand_index = 'f'
                            strand_index_reverse = 'r'
                        if feature_strand == -1:
                            strand_index = 'r'
                            strand_index_reverse = 'f'

                        if (pos_start >= to_plot_region[0]) and (pos_end <= to_plot_region[1]):
                            pos_start_new = pos_start - to_plot_region[0]
                            pos_end_new   = pos_end - to_plot_region[0]

                            if reverse_feature is False:
                                to_write_feature_list.append('%s,%s,%s,%s,%s,%s' % (location_label, to_label, strand_index, pos_start_new, pos_end_new, gene_color))
                            else:
                                pos_start_new_reversed = to_plot_region_len - pos_end_new + 1
                                pos_end_new_reversed   = to_plot_region_len - pos_start_new + 1
                                to_write_feature_list.append('%s,%s,%s,%s,%s,%s' % (location_label, to_label, strand_index_reverse, pos_start_new_reversed, pos_end_new_reversed, gene_color))

    return to_write_feature_list


def read_in_ctg_feature(ctg_feature_txt):

    ctg_pos_dict = dict()
    ctg_feature_dict = dict()
    for each_gene in open(ctg_feature_txt):
        gene_split = each_gene.strip().split(',')
        ctg_id = gene_split[0]
        if ctg_id not in ctg_feature_dict:
            ctg_feature_dict[ctg_id] = []
            ctg_pos_dict[ctg_id] = set()

        gene_id = gene_split[1]
        if gene_id == '':
            gene_id = ' '
        gene_start = int(gene_split[3])
        gene_end = int(gene_split[4])
        gene_color = gene_split[5]
        if gene_color == '':
            gene_color = "#FFFFCC"

        ctg_pos_dict[ctg_id].add(gene_start)
        ctg_pos_dict[ctg_id].add(gene_end)

        strand_index = ''
        if gene_split[2] == 'f':
            strand_index = +1
        elif gene_split[2] == 'r':
            strand_index = -1
        else:
            print('Strand value not valid for %s!, specify as "f" or "r".' % gene_id)
            exit()

        ctg_feature_dict[ctg_id].append(GraphicFeature(start=gene_start, end=gene_end, strand=strand_index, color=gene_color, label=gene_id))

    ctg_range_dict = {i: [min(ctg_pos_dict[i]), max(ctg_pos_dict[i])] for i in ctg_pos_dict}

    return ctg_feature_dict, ctg_range_dict


def plot_ctg_feature_txt(ctg_feature_txt, hide_title, hide_ruler, plot_height, plot_width, out_plot):

    ctg_feature_dict, ctg_range_dict = read_in_ctg_feature(ctg_feature_txt)
    ctg_list_sorted = sorted([i for i in ctg_feature_dict])

    if len(ctg_list_sorted) == 1:
        ctg_id = ctg_list_sorted[0]
        ctg_feature_list = ctg_feature_dict[ctg_id]
        ctg_range_list = ctg_range_dict[ctg_id]
        graph_record = GraphicRecord(sequence_length=(ctg_range_list[1]), features=ctg_feature_list)
        ax, _ = graph_record.plot(figure_width=5)
        ax.figure.savefig(out_plot)
    else:
        fig, ax = plt.subplots(nrows=len(ctg_feature_dict), ncols=1, figsize=(plot_width, len(ctg_feature_dict)*plot_height*1.2))
        i = 0
        for each_ctg in ctg_list_sorted:
            ctg_feature_list = ctg_feature_dict[each_ctg]
            ctg_range_list = ctg_range_dict[each_ctg]
            graph_record = GraphicRecord(sequence_length=(ctg_range_list[1]), features=ctg_feature_list)

            if hide_title is False:
                ax[i].set_xlabel(each_ctg)

            graph_record.plot(ax=ax[i], with_ruler=(not hide_ruler))
            i += 1

        #plt.subplots_adjust(hspace=plot_height*0.1)
        plt.savefig(out_plot)


def DnaFeaturesViewer(args):

    input_gbk                 = args['gbk']
    gbk_file_ext            = args['x']
    target_to_plot          = args['tgt']
    plot_by_gene            = args['by_gene']
    plot_by_fun             = args['by_fun']
    gene_fun_txt            = args['gf']
    gene_or_fun_color_txt   = args['c']
    output_plot             = args['o']
    flk_len_to_plot         = args['l']
    plot_width              = args['width']
    plot_height             = args['height']
    no_title                = args['no_title']
    no_ruler                = args['no_ruler']
    forward_strand_target   = args['ft']

    seq_feature_csv = '%s.%s.csv' % (output_plot, flk_len_to_plot)

    # check input gbk file
    gbk_file_list = []
    if os.path.isdir(input_gbk) is True:
        gbk_file_re = '%s/*.%s' % (input_gbk, gbk_file_ext)
        gbk_file_list = glob.glob(gbk_file_re)
        if len(gbk_file_list) == 0:
            print('gbk file not found in %s. program exited!' % input_gbk)
            exit()
    elif os.path.isfile(input_gbk) is True:
        gbk_file_list = [input_gbk]
    else:
        print('gbk file not found. program exited!')
        exit()

    # read in target_to_plot
    target_gene_dict = dict()
    target_fun_dict = dict()
    if plot_by_gene is True:
        if os.path.isdir(input_gbk) is True:
            if os.path.isfile(target_to_plot) is True:
                for each_g in open(target_to_plot):
                    each_g_split = each_g.strip().split('\t')
                    gnm_id = each_g_split[0]
                    gnm_target_list = each_g_split[1].split(',')
                    pwd_gbk = '%s/%s.%s' % (input_gbk, gnm_id, gbk_file_ext)
                    target_gene_dict[pwd_gbk] = gnm_target_list
            else:
                print('Please provide to-plot target genes for all genomes in a single txt file, please separates genome (without extension) and target(s) with tab, separates targets from the same genome with comma.')
                exit()
        elif os.path.isfile(input_gbk) is True:
            gbk_path, gbk_base, gbk_ext = sep_path_basename_ext(input_gbk)
            if os.path.isfile(target_to_plot) is True:
                for each_g in open(target_to_plot):
                    each_g_split = each_g.strip().split('\t')
                    gnm_id = each_g_split[0]
                    gnm_target_list = each_g_split[1].split(',')
                    if gnm_id == gbk_base:
                        target_gene_dict[input_gbk] = gnm_target_list
            else:
                target_list = target_to_plot.split(',')
                target_gene_dict[input_gbk] = target_list
    elif plot_by_fun is True:
        target_list = target_to_plot.split(',')
        for each_gbk in gbk_file_list:
            target_fun_dict[each_gbk] = target_list
    else:
        print('Please specify either "-by_gene" or "-by_fun"')


    # read in gene function
    gene_fun_dict = dict()
    if gene_fun_txt is not None:
        if os.path.isfile(gene_fun_txt) is True:
            for each_gene in open(gene_fun_txt):
                each_gene_split = each_gene.strip().split('\t')
                gene_fun_dict[each_gene_split[0]] = each_gene_split[1]
        else:
            print('%s not found, program exited!' % gene_fun_txt)
            exit()

    # read in gene/function color
    fun_color_dict = dict()
    if gene_or_fun_color_txt is not None:
        if os.path.isfile(gene_or_fun_color_txt) is True:
            for each_fun in open(gene_or_fun_color_txt):
                each_fun_split = each_fun.strip().split('\t')
                fun_color_dict[each_fun_split[0]] = each_fun_split[1]
        else:
            print('%s not found, program exited!' % gene_or_fun_color_txt)
            exit()

    gnm_to_gene_dict = target_gene_dict
    if plot_by_fun is True:
        gnm_to_gene_dict = dict()
        for each_gbk in gbk_file_list:
            gnm_fun_to_plot = target_fun_dict.get(each_gbk)
            for seq_record in SeqIO.parse(each_gbk, 'genbank'):
                for seq_feature in seq_record.features:
                    if seq_feature.type == "CDS":
                        locus_tag = seq_feature.qualifiers['locus_tag'][0]
                        gene_fun = gene_fun_dict.get(locus_tag, 'na')
                        if plot_by_fun is True:
                            if gene_fun in gnm_fun_to_plot:
                                if each_gbk not in gnm_to_gene_dict:
                                    gnm_to_gene_dict[each_gbk] = {locus_tag}
                                else:
                                    gnm_to_gene_dict[each_gbk].add(locus_tag)

    print('target_gene_dict\t%s'    % target_gene_dict)
    print('target_fun_dict\t\t%s'   % target_fun_dict)
    print('gene_fun_dict\t\t%s'     % gene_fun_dict)
    print('fun_color_dict\t\t%s'    % fun_color_dict)
    print('gnm_to_gene_dict\t%s'  % gnm_to_gene_dict)

    # write out features
    print('Extracting features to plot')
    with open(seq_feature_csv, 'w') as seq_feature_csv_handle:
        for each_gnm in gnm_to_gene_dict:
            pwd_gbk = each_gnm
            locus_tag_to_plot = gnm_to_gene_dict[each_gnm]
            features_to_write_list = gbk_to_feature_list(pwd_gbk, locus_tag_to_plot, flk_len_to_plot, forward_strand_target, fun_color_dict)
            seq_feature_csv_handle.write('\n'.join(features_to_write_list) + '\n')

    # plot
    print('Getting plot')
    plot_ctg_feature_txt(seq_feature_csv, no_title, no_ruler, plot_height, plot_width, output_plot)

    print('Plot exported to %s' % output_plot)
    print('Done!')


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-gbk',         required=True,                          help='gbk file')
    parser.add_argument('-x',           required=False,                         default='gbk', help='file extension, default: gbk')
    parser.add_argument('-tgt',         required=True,                          help='targets to plot')
    parser.add_argument('-by_gene',     required=False, action='store_true',    help='target plot region by gene id (locus_tag)')
    parser.add_argument('-by_fun',      required=False, action='store_true',    help='target plot region by gene function')
    parser.add_argument('-gf',          required=False, default=None,           help='gene to function file, tab separated')
    parser.add_argument('-c',           required=False, default=None,           help='gene/function to color file, tab separated')
    parser.add_argument('-o',           required=True,                          help='output plot')
    parser.add_argument('-l',           required=False, type=int, default=7500, help='length (in bp) of flanking sequences to plot')
    parser.add_argument('-width',       required=False, type=int, default=22,   help='plot_width, default: 22')
    parser.add_argument('-height',      required=False, type=int, default=2,    help='subplot_height, default: 2')
    parser.add_argument('-no_title',    required=False, action='store_true',    help='no_title')
    parser.add_argument('-no_ruler',    required=False, action='store_true',    help='no_ruler')
    parser.add_argument('-ft',          required=False, action='store_true',    help='forward target strand')
    args = vars(parser.parse_args())
    DnaFeaturesViewer(args)

'''
cd /Users/songweizhi/Desktop/Brady/draw_shc_wd
python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/DnaFeaturesViewer.py -gbk gbk_dir -x gbk -tgt  -by_gene -o demo.pdf

gene_to_fun.txt
fun_to_color.txt

# single gbk
python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/DnaFeaturesViewer.py -gbk gbk_dir/Bradyrhizobium_oligotrophicum_SZCCHNR1093.gbk -by_gene -o demo_by_g.pdf -tgt SZCCHNR1093_01706,SZCCHNR1093_01729
python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/DnaFeaturesViewer.py -gbk gbk_dir/Bradyrhizobium_oligotrophicum_SZCCHNR1093.gbk -by_gene -o demo_by_g.pdf -tgt targeted_gene.txt

# multiple gbk
python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/DnaFeaturesViewer.py -gbk gbk_dir -x gbk -by_gene -o demo_by_g.pdf -tgt targeted_gene.txt





python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/DnaFeaturesViewer.py -gbk gbk_dir -x gbk -by_fun -o demo_by_f.pdf -tgt shc,mntB -c fun_color.txt
python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/DnaFeaturesViewer.py -gbk gbk_dir/Bradyrhizobium_oligotrophicum_SZCCHNR1091.gbk -by_fun -o demo_by_f.pdf -tgt shc,mntB -c fun_color.txt

single file/multi file

gene by str (for single file)
gene by file (for multi file)

fun by str
fun by file


'''



import os
import glob
import argparse
import multiprocessing as mp


sra_usage = '''
============ sra example commands ============

Requirement: sratoolkit
BioSAK sra -i sra_id.txt -o op_dir -t 12

# the format of input file (tab separated)
SRS5161477	IR1T0
SRS5161479	IR2T0

==============================================
'''


def sra(args):

    id_txt              = args['i']
    op_dir              = args['o']
    num_threads         = args['t']
    force_create_op_dir = args['f']
    run_dump            = args['dump']
    max_size            = args['maxsize']

    fasterq_dump_tmp    = '%s/fasterq_dump_tmp' % op_dir
    sra_dir             = '%s/sra_dir'          % op_dir
    cmd_txt             = '%s/cmds.txt'         % op_dir

    # create output folder
    if os.path.isdir(op_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s' % sra_dir)

    id_to_desc_dict = dict()
    id_set = set()
    for each_id in open(id_txt):
        each_id_split = each_id.strip().split()
        sra_id = each_id_split[0]
        sra_desc = sra_id
        if len(each_id_split) >= 2:
            sra_desc = each_id_split[1]
        id_to_desc_dict[sra_id] = sra_desc
        id_set.add(sra_id)

    prefetch_cmd_list = []
    fasterq_dump_cmd_list = []

    for each_id in sorted(list(id_set)):
        customized_name = id_to_desc_dict.get(each_id, each_id)
        mkdir_cmd        = 'mkdir %s/%s'                                    % (op_dir, customized_name)
        if max_size == '20G':
            prefetch_cmd     = 'prefetch %s -O %s/%s'                       % (each_id, op_dir, customized_name)
        else:
            prefetch_cmd     = 'prefetch %s -O %s/%s --max-size %s'         % (each_id, op_dir, customized_name, max_size)

        fasterq_dump_cmd = 'fasterq-dump %s/%s.sra --split-3 -O %s -t %s' % (sra_dir, each_id, op_dir, fasterq_dump_tmp)
        prefetch_cmd_list.append(prefetch_cmd)
        fasterq_dump_cmd_list.append(fasterq_dump_cmd)
        os.system(mkdir_cmd)

    # write out commands
    cmd_txt_handle = open(cmd_txt, 'w')
    for each in sorted(prefetch_cmd_list):
        cmd_txt_handle.write(each + '\n')
    for each in sorted(fasterq_dump_cmd_list):
        cmd_txt_handle.write(each + '\n')
    cmd_txt_handle.close()

    # download files with multiprocessing
    print('Running prefetch')
    pool = mp.Pool(processes=num_threads)
    pool.map(os.system, prefetch_cmd_list)
    pool.close()
    pool.join()

    for each_id in id_set:
        customized_name = id_to_desc_dict.get(each_id, each_id)
        sra_file_re = '%s/%s/*/*.sra' % (op_dir, customized_name)
        sra_file    = glob.glob(sra_file_re)[0]
        os.system('mv %s %s/%s.sra' % (sra_file, sra_dir, customized_name))
        os.system('rm -r %s/%s' % (op_dir, customized_name))

    # extract fastq file from SRA with multiprocessing
    if run_dump is True:
        print('Running fasterq-dump')
        os.system('mkdir %s' % fasterq_dump_tmp)
        pool = mp.Pool(processes=num_threads)
        pool.map(os.system, fasterq_dump_cmd_list)
        pool.close()
        pool.join()
        os.system('rm -r %s' % fasterq_dump_tmp)

    print('Done!')


if __name__ == '__main__':

    sra_parser = argparse.ArgumentParser()
    sra_parser.add_argument('-i',       required=True,                          help='id')
    sra_parser.add_argument('-o',       required=True,                          help='output directory')
    sra_parser.add_argument('-t',       required=False, type=int, default=1,    help='number of threads, default is 1')
    sra_parser.add_argument('-maxsize', required=False, default='20G',          help='--max-size for prefetch, default is 20G')
    sra_parser.add_argument('-dump',    required=False, action="store_true",    help='run fasterq-dump')
    sra_parser.add_argument('-f',       required=False, action="store_true",    help='force overwrite')
    args = vars(sra_parser.parse_args())
    sra(args)



import os
import argparse


srun_usage = '''
=============== srun example commands ===============

BioSAK srun -c "cd clock2_ns100000_run1; mcmctree"
BioSAK srun -c "cd clock2_ns100000_run2; mcmctree"

=====================================================
'''


def srun(args):

    cmd_to_run = args['c']

    cmd_to_run_split = cmd_to_run.strip().split(';')

    for each_cmd in cmd_to_run_split:
        each_cmd = each_cmd.strip()
        if each_cmd.startswith('cd '):
            os.chdir(each_cmd[3:])
        else:
            os.system(each_cmd)


if __name__ == '__main__':

    srun_parser = argparse.ArgumentParser()
    srun_parser.add_argument('-c', required=True,  help='command to run')
    args = vars(srun_parser.parse_args())
    srun(args)



import os
import argparse
import pandas as pd


rm_low_abd_otu_usage = '''
=================== rm_low_abd_otu example commands ===================

# ignore OTUs with relative abundance lower than 0.1%
BioSAK rm_low_abd_otu -i otu_table.txt -c 0.001 -o otu_table_0.001.txt

# ignore OTUs with relative abundance lower than 1%
BioSAK rm_low_abd_otu -i otu_table.txt -c 0.01 -o otu_table_0.01.txt

=======================================================================
'''


def transpose_csv(file_in, file_out, sep_symbol, column_name_pos, row_name_pos):
    csv = pd.read_csv(file_in, sep=sep_symbol, header=column_name_pos, index_col=row_name_pos)
    df_csv = pd.DataFrame(data=csv)
    transposed_csv = df_csv.T
    transposed_csv.to_csv(file_out, sep=sep_symbol)


def rm_low_abd_otu(args):

    otu_table_in    = args['i']
    abd_cutoff      = args['c']
    otu_table_out   = args['o']

    # define tmp file name
    tmp1_t          = '%s.tmp1' % otu_table_out
    tmp2_t_filtered = '%s.tmp2' % otu_table_out

    transpose_csv(otu_table_in, tmp1_t, '\t', 0, 0)

    otu_table_txt_t_filtered_handle = open(tmp2_t_filtered, 'w')
    line_index = 0
    for each_line in open(tmp1_t):
        if line_index == 0:
            otu_table_txt_t_filtered_handle.write(each_line)
        else:
            each_line_split = each_line.strip().split('\t')
            sample_id = each_line_split[0]
            count_list = [int(i) for i in each_line_split[1:]]
            count_sum = sum(count_list)
            count_list_filtered = []
            for each_count in count_list:
                if (each_count/count_sum) >= abd_cutoff:
                    count_list_filtered.append(str(each_count))
                else:
                    count_list_filtered.append('0')
            otu_table_txt_t_filtered_handle.write('%s\t%s\n' % (sample_id, '\t'.join(count_list_filtered)))
        line_index += 1
    otu_table_txt_t_filtered_handle.close()

    transpose_csv(tmp2_t_filtered, otu_table_out, '\t', 0, 0)

    # remove tmp files
    os.remove(tmp1_t)
    os.remove(tmp2_t_filtered)
    print('Done')


if __name__ == '__main__':

    rm_low_abd_otu_parser = argparse.ArgumentParser()
    rm_low_abd_otu_parser.add_argument('-i', required=True,                              help='input otu count table')
    rm_low_abd_otu_parser.add_argument('-c', required=False, default=0.001,type=float,   help='relative abundance cutoff, default is 0.001')
    rm_low_abd_otu_parser.add_argument('-o', required=True,                              help='output otu count table')
    args = vars(rm_low_abd_otu_parser.parse_args())
    rm_low_abd_otu(args)
